<?php

namespace Drupal\static_export_graphql_resolver\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\static_export\Entity\ExportableEntityInterface;
use Drupal\static_export\Entity\ExportableEntityManagerInterface;
use Drupal\static_export\Event\StaticExportEvents;
use Drupal\static_export\Event\StaticExportEvent;
use Drupal\static_export\Plugin\ResolverManagerInterface;
use Drupal\static_suite\Entity\EntityUtils;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriber for GraphQL Variants.
 */
class VariantsEventSubscriber implements EventSubscriberInterface {

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Resolver manager.
   *
   * @var \Drupal\static_export\Plugin\ResolverManagerInterface
   */
  protected $resolverManager;

  /**
   * Exportable entity manager.
   *
   * @var \Drupal\static_export\Entity\ExportableEntityManagerInterface
   */
  protected $exportableEntityManager;

  /**
   * Entity Utils.
   *
   * @var \Drupal\static_suite\Entity\EntityUtils
   */
  protected $entityUtils;

  /**
   * Constructs the EntityEventSubscriber object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\static_export\Plugin\ResolverManagerInterface $resolverManager
   * @param \Drupal\static_export\Entity\ExportableEntityManagerInterface $exportableEntityManager
   *   Exportable entity manager.
   * @param \Drupal\static_suite\Entity\EntityUtils $entityUtils
   *   Utils for working with entities.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ResolverManagerInterface $resolverManager, ExportableEntityManagerInterface $exportableEntityManager, EntityUtils $entityUtils) {
    $this->configFactory = $config_factory;
    $this->resolverManager = $resolverManager;
    $this->exportableEntityManager = $exportableEntityManager;
    $this->entityUtils = $entityUtils;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[StaticExportEvents::WRITE_STARTS][] = ['onWriteStarts'];
    return $events;
  }

  /**
   * Reacts to a StaticExportEvents::WRITE_STARTS event.
   *
   * @param \Drupal\static_export\Event\StaticExportEvent $event
   *   The Static Export event.
   *
   * @return \Drupal\static_export\Event\StaticExportEvent
   *   The processed event.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function onWriteStarts(StaticExportEvent $event) {
    $eventExporter = $event->getExporter();
    if ($eventExporter->getPluginId() === 'entity') {
      $entity = $eventExporter->getExporterItem();

      // Search referenced entities only for the master export (the first one)
      // and if it's configured to do that
      // and is not in standalone mode
      // and it's not an unpublished entity.
      if ($eventExporter->isMasterExport()) {
        $exportableEntity = $this->exportableEntityManager->getExportableEntity($entity);
        if ($exportableEntity && $exportableEntity->status() && $exportableEntity->getResolver() === self::getSupportedResolverId()) {
          $variants = $this->getVariants($exportableEntity);
          if (count($variants) > 0) {
            $fileCollection = $this->exportVariants($eventExporter, $variants);
            $event->getFileCollection()->merge($fileCollection);
          }
        }
      }
    }
    return $event;
  }

  /**
   * Exports entities referenced by the original one.
   *
   * @param \Drupal\static_export\Plugin\ExporterInterface $entityExporter
   *   The event entity exporter.
   *
   * @return \Drupal\static_export\File\FileCollection
   *   A file collection.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\static_suite\StaticSuiteUserException
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  protected function exportVariants(ExporterInterface $entityExporter): FileCollection {
    $fileCollection = new FileCollection($entityExporter->uniqueId());
    $entityBeingExported = $entityExporter->getExporterItem();
    $exportableEntity = $this->exportableEntityManager->getExportableEntity($entityBeingExported);
    if ($exportableEntity) {
      $relatedEntities = $this->entityReferenceFinder->findReferences(
      $entityBeingExported,
      $exportableEntity->getRecursionLevel()
      );
      foreach ($relatedEntities as $relatedEntity) {
        // Avoid exporting a related entity that is not exportable.
        if (!$this->exportableEntityManager->isExportable($relatedEntity)) {
          continue;
        }

        // We need a new instance, so create a new one instead of getting it.
        $exporter = $this->exporterManager->createInstance('entity');
        $exporter->makeSlaveOf($entityExporter);
        $relatedEntitiesFileCollectionGroup = $exporter->export(
        [
          $exporter::PARAM_ENTITY => $relatedEntity,
        ],
        TRUE,
        $entityExporter->mustLogToFile(),
        $entityExporter->isLock()
        );

        $fileCollection->mergeMultiple($relatedEntitiesFileCollectionGroup->getFileCollections());
      }
    }
    return $fileCollection;
  }



  /**
   * Tells the id of the resolver that is supported by this event subscriber.
   *
   * It's in a separate method to allow easy overriding from other modules that
   * could implement variants.
   *
   * @return string
   */
  protected static function getSupportedResolverId(): string {
    return 'graphql';
  }

}
