<?php

namespace Drupal\static_export_graphql_resolver\Plugin\Resolver;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\static_export_graphql_resolver\GraphqlQueryHandlerInterface;
use Drupal\static_suite\Entity\EntityUtils;
use Drupal\static_export\Plugin\ResolverInterface;
use Drupal\static_suite\StaticSuiteUserException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A GraphQL resolver.
 *
 * @StaticResolver(
 *  id = "graphql",
 *  label = "GraphQL",
 *  description = "GraphQL data resolver",
 * )
 */
class GraphqlResolver extends PluginBase implements ResolverInterface, ContainerFactoryPluginInterface {

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Entity Utils.
   *
   * @var \Drupal\static_suite\Entity\EntityUtils
   */
  protected $entityUtils;

  /**
   * Handler for querying GraphQL.
   *
   * @var \Drupal\static_export_graphql_resolver\GraphqlQueryHandlerInterface
   */
  protected $graphqlQueryHandler;

  /**
   * Constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\static_suite\Entity\EntityUtils $entityUtils
   *   Entity utils service.
   * @param \Drupal\static_export_graphql_resolver\GraphqlQueryHandlerInterface $graphqlQueryHandler
   *   Service for querying GraphQL.
   */
  public function __construct(array $configuration,
  string $plugin_id,
  $plugin_definition,
  ConfigFactoryInterface $config_factory,
  EntityUtils $entityUtils,
  GraphqlQueryHandlerInterface $graphqlQueryHandler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
    $this->entityUtils = $entityUtils;
    $this->graphqlQueryHandler = $graphqlQueryHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get("config.factory"),
      $container->get("static_suite.entity_utils"),
      $container->get("static_export_graphql_resolver.graphql_query_handler")
    );
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function resolve(EntityInterface $entity, string $variant = NULL): array {
    $query = NULL;
    /** @var $exportableEntity \Drupal\static_export\Entity\ExportableEntity */
    $exportableEntity = $this->entityUtils->loadEntity('exportable_entity', $entity->getEntityTypeId() . '.' . $entity->bundle());
    if ($exportableEntity && $exportableEntity->status()) {
      $graphqlFile = $this->configFactory->get('static_export_graphql_resolver.settings')
        ->get('dir') . '/' . $exportableEntity->id() . ($variant ? '.' . $variant : '') . '.gql';
      if (is_file($graphqlFile)) {
        $query = file_get_contents($graphqlFile);
      }
      else {
        throw new StaticSuiteUserException("Resolver query file located at '" . $graphqlFile . "' not found.");
      }
    }
    else {
      throw new StaticSuiteUserException("ExportableEntity for entity " . $entity->id() . " not found.");
    }

    $variables = [
      'entityId' => $entity->id(),
      'uuid' => $entity->uuid(),
      'language' => strtoupper(str_replace('-', '_', $entity->language()->getId())),
    ];

    // Throw an error if query is using nodeById without a "content" alias.
    if (strpos($query, 'nodeById(') !== FALSE && !preg_match("/content:\s*nodeById\(/", $query)) {
      throw new StaticSuiteUserException("Resolver query file located at '" . $graphqlFile . "' is using nodeById() query without a 'content' alias. It should be aliased this way: 'content:nodeById(...) { ... }'");
    }

    $result = $this->graphqlQueryHandler->query($query, $variables);

    // Variants are pieces of data that are later included inside other files,
    // so we remove "data" and "content" outer structures.
    if ($variant && !empty($result['data']['content'])) {
      return $result['data']['content'];
    }
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getVariantKeys(EntityInterface $entity): array {
    $variantKeys = [];
    /** @var $exportableEntity \Drupal\static_export\Entity\ExportableEntity */
    $exportableEntity = $this->entityUtils->loadEntity('exportable_entity', $entity->getEntityTypeId() . '.' . $entity->bundle());
    if ($exportableEntity && $exportableEntity->status()) {
      $basePath = $this->configFactory->get('static_export_graphql_resolver.settings')
        ->get('dir') . '/' . $exportableEntity->id() . '.';
      $extension = '.gql';
      $pattern = $basePath . '*' . $extension;
      $files = glob($pattern);
      foreach ($files as $file) {
        $variantFilename = str_replace($basePath, '', $file);
        $variantKey = pathinfo($variantFilename, PATHINFO_FILENAME);
        $variantKeys[] = $variantKey;
      }
    }
    return $variantKeys;
  }

}
