<?php

namespace Drupal\static_export\Commands;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\static_export\File\FileCollectionFormatter;
use Drupal\static_export\Plugin\ExporterManagerInterface;
use Drupal\static_suite\StaticSuiteUserException;
use Drush\Commands\DrushCommands;
use Throwable;

/**
 * A Drush command file to export drupal config data.
 */
class ConfigCommands extends DrushCommands {

  /**
   * Drupal's config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Exporter Manager.
   *
   * @var \Drupal\static_export\Plugin\ExporterManagerInterface
   */
  protected $exporterManager;

  /**
   * Entity Utils.
   *
   * @var \Drupal\static_export\File\FileCollectionFormatter
   */
  protected $fileCollectionFormatter;

  /**
   * StaticExportCommands constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Drupal's config factory.
   * @param \Drupal\static_export\Plugin\ExporterManagerInterface $exporterManager
   *   Exporter Manager.
   * @param \Drupal\static_export\File\FileCollectionFormatter $file_collection_formatter
   *   File collection formatter.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ExporterManagerInterface $exporterManager, FileCollectionFormatter $file_collection_formatter) {
    parent::__construct();
    $this->configFactory = $config_factory;
    $this->exporterManager = $exporterManager;
    $this->fileCollectionFormatter = $file_collection_formatter;
  }

  /**
   * Exports Drupal configuration objects.
   *
   * @param string $configName
   *   The name of the Drupal config to export. Optional.
   * @param array $execOptions
   *   An associative array of options whose values come
   *   from cli, aliases, config, etc.
   *
   * @command static-export:export-config
   *
   * @option standalone Flag to set standalone mode
   * @usage drush static-export:export-config pt-br
   *   Exports drupal config
   * @aliases sedc, static-export-config
   *
   * @static_export Annotation for drush hooks.
   */
  public function exportConfig(
    $configName = NULL,
    array $execOptions = [
      'standalone' => FALSE,
      'log-to-file' => TRUE,
      'lock' => TRUE,
      'build' => FALSE,
    ]
  ) {
    $timeStart = microtime(TRUE);
    try {
      if (!empty($configName)) {
        $exportableConfigNames[] = $configName;
      }
      else {
        $configExporter = $this->exporterManager->getInstance(['plugin_id' => 'config']);
        $exportableConfigNames = $this->configFactory->get('static_export.settings')->get('exportable_config.objects_to_export');
      }
      $delta = 1;
      $total = count($exportableConfigNames);
      $configExporter->setMustBuild($execOptions['build']);
      foreach ($exportableConfigNames as $exportableConfigName) {
        $fileCollectionGroup = $configExporter->export(
          [
            $configExporter::PARAM_CONFIG_NAME => $exportableConfigName,
          ],
          $execOptions['standalone'],
          $execOptions['log-to-file'],
          $execOptions['lock']
        );
        foreach ($fileCollectionGroup->getFileCollections() as $fileCollection) {
          $this->fileCollectionFormatter->setFileCollection($fileCollection);
          $this->output()->writeln($this->fileCollectionFormatter->getTextLines($delta, $total));
        }
        $delta++;
      }
      $this->output()->writeln("TOTAL TIME: " . (round(microtime(TRUE) - $timeStart, 3)) . " secs.");
    }
    catch (StaticSuiteUserException | PluginException $e) {
      $this->logger()->error($e->getMessage());
    }
    catch (Throwable $e) {
      $this->logger()->error($e);
    }
  }

}
