<?php

namespace Drupal\static_export\Commands;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\static_export\Plugin\ExporterInterface;
use Drupal\static_export\Semaphore\SemaphoreProvider;
use Drupal\static_export\Semaphore\SemaphoreService;
use Drupal\static_suite\StaticSuiteUserException;
use Drush\Commands\DrushCommands;
use Throwable;

/**
 * A Drush command file to copy data dir.
 */
class CopyDataCommands extends DrushCommands {

  /**
   * Drupal's config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * A System V semaphore service.
   *
   * @var \Drupal\static_export\Semaphore\SemaphoreService
   */
  protected $semaphoreService;

  /**
   * Semaphore provider.
   *
   * @var \Drupal\static_export\Semaphore\SemaphoreProvider
   */
  protected $semaphoreProvider;

  /**
   * StaticExportCopyDataCommands constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Drupal's config factory.
   * @param \Drupal\static_export\Semaphore\SemaphoreService $semaphore_service
   *   Semaphore service.
   * @param \Drupal\static_export\Semaphore\SemaphoreProvider $semaphore_provider
   *   Semaphore provider.
   */
  public function __construct(ConfigFactoryInterface $config_factory, SemaphoreService $semaphore_service, SemaphoreProvider $semaphore_provider) {
    parent::__construct();
    $this->configFactory = $config_factory;
    $this->semaphoreService = $semaphore_service;
    $this->semaphoreProvider = $semaphore_provider;
  }

  /**
   * Copy data folder on given path.
   *
   * @param string $target
   *   Destination folder.
   * @param array $execOptions
   *   An associative array of options whose values come
   *   from cli, aliases, config, etc.
   *
   * @command static-export:copy-data-dir
   *
   * @option tar target tar file
   * @usage drush static-export:copy-data-dir /data
   *   Copy data folder to /data dir
   * @usage drush static-export:copy-data-dir /data --tar=/data.tar
   *   Copy data folder to /data and tar it into /data.tar.
   * @usage drush static-export:copy-data-dir /data --tar=/data.tar --gzip
   *   Copy data folder to /data, tar it into /data.tar, and gzip it.
   * @aliases secdd, static-export-copy-data-dir
   *
   * @static_export Annotation for drush hooks.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function copyDataDir(
    $target,
    array $execOptions = [
      'tar' => NULL,
      'gzip' => FALSE,
    ]
  ) {

    // Check --gzip validity.
    if (!empty($execOptions['gzip']) && empty($execOptions['tar'])) {
      throw new StaticSuiteUserException("--gzip option requires --tar option");
    }

    // Check --tar validity.
    if (!empty($execOptions['tar']) && !is_string($execOptions['tar'])) {
      throw new StaticSuiteUserException("--tar option must be a path");
    }

    $timeStart = microtime(TRUE);
    $dataDir = $this->configFactory->get("static_export.settings")->get("data_dir");
    $logDir = $this->configFactory->get("static_export.settings")->get("log_dir");

    // SemaphoreService::create() needs $target to be already present.
    if (!is_dir($target)) {
      mkdir($target, 0777, TRUE);
    }
    $this->output()->write("Acquiring lock on target dir: ");
    $targetDirSemaphore = $this->semaphoreService->create('target-data-dir', $target, "c");
    $targetDirSemaphore->acquire();
    $this->output()->writeln("DONE");

    $dataDirSemaphore = $this->semaphoreProvider->createDataDirSemaphore();
    $this->output()->write("Acquiring lock on data dir: ");
    $dataDirSemaphore->acquire();
    $this->output()->writeln("DONE");

    try {
      if (!is_writable($target)) {
        throw new StaticSuiteUserException($target . " is not writable");
      }
      if (is_file($target)) {
        throw new StaticSuiteUserException($target . " is a file");
      }
      $exclude = " --exclude='" . ExporterInterface::WORK_DIR . "'";
      if (strpos($logDir, $dataDir) !== FALSE) {
        $exclude .= " --exclude='" . str_replace($dataDir . '/', '', $logDir) . "'";
      }
      $command = "rsync -av$exclude " . $dataDir . "/ $target --delete";
      $this->output()->write("Copying " . $dataDir . "/ to " . $target . ": ");
      exec($command, $output, $returnValue);
      if ($returnValue !== 0) {
        throw new StaticSuiteUserException("Unable to copy " . $dataDir . " to $target:\n" . implode("\n", $output));
      }
      $this->output()->writeln("DONE");
      $dataDirSemaphore->release();
      $this->output()->writeln("Lock released on data dir.");

      $tarFile = $execOptions['tar'];
      if (!empty($tarFile)) {
        $this->output()->write("Archiving in " . $tarFile . ": ");
        if (!is_dir(dirname($tarFile))) {
          mkdir(dirname($tarFile), 0777, TRUE);
        }

        if (!is_writable(dirname($tarFile))) {
          throw new StaticSuiteUserException(dirname($tarFile) . " is not writable");
        }

        if (is_file($tarFile) && !is_writable($tarFile)) {
          throw new StaticSuiteUserException($tarFile . " is not writable");
        }

        $tarOptions = $execOptions['gzip'] ? "zcf" : "cf";
        $command = "cd $target && tar -$tarOptions " . $tarFile . " *";
        exec($command, $output, $returnValue);
        if ($returnValue !== 0) {
          throw new StaticSuiteUserException("Unable to archive " . $target . " in " . $tarFile . ":\n" . implode("\n", $output));
        }
        $this->output()->writeln("DONE");
      }
    }
    catch (StaticSuiteUserException $e) {
      $this->logger()->error($e->getMessage());
    }
    catch (Throwable $e) {
      $this->logger()->error($e);
    }
    finally {
      if ($dataDirSemaphore->isAcquired()) {
        $dataDirSemaphore->release();
        $this->output()->writeln("Lock released on data dir.");
      }
      $targetDirSemaphore->release();
      $this->output()->writeln("Lock released on target dir.");
    }

    $this->output()->writeln("TOTAL TIME: " . (round(microtime(TRUE) - $timeStart, 3)) . " secs.");
  }

}
