<?php

namespace Drupal\static_export\Commands;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\static_export\File\FileCollectionFormatter;
use Drupal\static_export\Plugin\ExporterManagerInterface;
use Drupal\static_suite\StaticSuiteUserException;
use Drush\Commands\DrushCommands;
use Throwable;

/**
 * A Drush command file to export translation data.
 */
class TranslationCommands extends DrushCommands {

  /**
   * Language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Exporter Manager.
   *
   * @var \Drupal\static_export\Plugin\ExporterManagerInterface
   */
  protected $exporterManager;

  /**
   * Entity Utils.
   *
   * @var \Drupal\static_export\File\FileCollectionFormatter
   */
  protected $fileCollectionFormatter;

  /**
   * StaticExportCommands constructor.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager.
   * @param \Drupal\static_export\Plugin\ExporterManagerInterface $exporterManager
   *   Exporter Manager.
   * @param \Drupal\static_export\File\FileCollectionFormatter $file_collection_formatter
   *   File collection formatter.
   */
  public function __construct(LanguageManagerInterface $language_manager, ExporterManagerInterface $exporterManager, FileCollectionFormatter $file_collection_formatter) {
    parent::__construct();
    $this->languageManager = $language_manager;
    $this->exporterManager = $exporterManager;
    $this->fileCollectionFormatter = $file_collection_formatter;
  }

  /**
   * Exports drupal translation.
   *
   * @param string $languageCode
   *   The language code to export. Optional. If none provided, all languages
   *   are exported.
   * @param array $execOptions
   *   An associative array of options whose values come
   *   from cli, aliases, config, etc.
   *
   * @command static-export:export-translation
   *
   * @option standalone Flag to set standalone mode
   * @usage drush static-export:export-translation pt-br
   *   Exports drupal translations.
   * @aliases set, static-export-translation
   *
   * @static_export Annotation for drush hooks.
   */
  public function exportTranslation(
    $languageCode = NULL,
    array $execOptions = [
      'standalone' => FALSE,
      'log-to-file' => TRUE,
      'lock' => TRUE,
      'build' => FALSE,
    ]
  ) {
    $timeStart = microtime(TRUE);

    $languageCodesToExport = [];
    if ($languageCode) {
      $languageCodesToExport[] = $languageCode;
    }
    else {
      foreach ($this->languageManager->getLanguages() as $language) {
        $languageCodesToExport[] = $language->getId();
      }
    }

    try {
      $translationExporter = $this->exporterManager->getInstance(['plugin_id' => 'translation']);
      foreach ($languageCodesToExport as $langCode) {
        $translationExporter->setMustBuild($execOptions['build']);
        $fileCollectionGroup = $translationExporter->export(
          [$translationExporter::PARAM_LANG_CODE => $langCode],
          $execOptions['standalone'],
          $execOptions['log-to-file'],
          $execOptions['lock']
        );
        foreach ($fileCollectionGroup->getFileCollections() as $fileCollection) {
          $this->fileCollectionFormatter->setFileCollection($fileCollection);
          $this->output()
            ->writeln($this->fileCollectionFormatter->getTextLines());
        }
      }
      $this->output()->writeln("TOTAL TIME: " . (round(microtime(TRUE) - $timeStart, 3)) . " secs.");
      // $this->entityExporter->logger()->info($message);
    }
    catch (StaticSuiteUserException | PluginException $e) {
      $this->logger()->error($e->getMessage());
    }
    catch (Throwable $e) {
      $this->logger()->error($e);
    }
  }

}
