<?php

namespace Drupal\static_export\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use finfo;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\PreconditionFailedHttpException;

/**
 * FileViewer controller.
 *
 * Provides a controller for viewing static export files..
 */
class FileViewerController extends ControllerBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The redirect destination helper.
   *
   * @var \Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $redirectDestination;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * FileViewer controller constructor.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Routing\RedirectDestinationInterface $redirect_destination
   *   The redirect destination helper.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    AccountProxyInterface $current_user,
    RedirectDestinationInterface $redirect_destination,
    ConfigFactoryInterface $config_factory
  ) {
    $this->currentUser = $current_user;
    $this->redirectDestination = $redirect_destination;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('redirect.destination'),
      $container->get('config.factory')
    );
  }

  /**
   * Controller to return a POST or a GET parameter.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
   *   The response.
   */
  public function viewPath(Request $request) {
    $dataDir = $this->configFactory->get('static_export.settings')->get('data_dir');
    if (empty($dataDir)) {
      throw new PreconditionFailedHttpException('static_export.settings.data_dir is empty');
    }
    $path = $request->get('path');

    if (!$path || !file_exists($path) || !is_readable($path)) {
      throw new NotFoundHttpException();
    }

    // Redirect to realpath (useful when viewing path with "/../")
    $realPath = realpath($path);
    if ($realPath !== $path) {
      $destination = Url::fromUserInput(
        $this->redirectDestination->get()
      );
      if ($destination->isRouted()) {
        // Valid internal path.
        return $this->redirect($destination->getRouteName(), ['path' => $realPath], [], 301);
      }
    }

    // Ensure $realPath starts with $dataDir.
    if (!preg_match("#^$dataDir#", $realPath)) {
      throw new NotFoundHttpException();
    }

    if (is_file($path)) {
      return $this->getResponseForFile($path);
    }

    throw new NotFoundHttpException();

  }

  /**
   * Get response for a file.
   *
   * @param string $file
   *   File to view.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response.
   */
  protected function getResponseForFile(string $file) {
    // Get mime type.
    $mimeType = $this->getMimeType($file);

    // Prepare response object.
    $response = new Response();
    $response->setContent(file_get_contents($file));
    $response->headers->set('Content-Type', $mimeType);

    // Return response object.
    return $response;
  }

  /**
   * Get mime type for a path.
   *
   * @param string $path
   *   Path to file.
   *
   * @return string
   *   The mime type.
   */
  private function getMimeType(string $path) {
    $extension = pathinfo($path, PATHINFO_EXTENSION);
    switch ($extension) {
      case 'json':
        return 'application/json';

      case 'xml':
        return 'text/xml';

      case 'yml':
        return 'text/yaml';

      default:
        $fileInfo = new finfo(FILEINFO_MIME_TYPE);
        return $fileInfo->file($path);
    }
  }

}
