<?php

namespace Drupal\static_export\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\static_export\Plugin\EntityExporterInterface;
use Drupal\static_export\Plugin\OutputFormatterManagerInterface;

/**
 * Defines the Exportable Entity.
 *
 * @ConfigEntityType(
 *   id = "exportable_entity",
 *   label = @Translation("Exportable entity"),
 *   handlers = {
 *     "list_builder" = "Drupal\static_export\Controller\ExportableEntityListBuilder",
 *     "form" = {
 *       "add" = "Drupal\static_export\Form\ExportableEntityForm",
 *       "edit" = "Drupal\static_export\Form\ExportableEntityForm",
 *       "delete" = "Drupal\static_export\Form\ExportableEntityDeleteForm",
 *       "enable" = "Drupal\static_export\Form\ExportableEntityEnableForm",
 *       "disable" = "Drupal\static_export\Form\ExportableEntityDisableForm"
 *     }
 *   },
 *   config_prefix = "exportable_entity",
 *   admin_permission = "administer site configuration",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *     "status" = "status"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "resolver",
 *     "format",
 *     "export_referencing_entities",
 *     "recursion_level",
 *     "export_when_crud_happens_on_cli",
 *     "request_build_when_crud_exports_on_cli"
 *   },
 *   links = {
 *     "edit-form" = "/admin/config/static/export/entity/{exportable_entity}",
 *     "delete-form" = "/admin/config/static/export/entity/{exportable_entity}/delete",
 *     "enable" = "/admin/config/static/export/entity/{exportable_entity}/enable",
 *     "disable" = "/admin/config/static/export/entity/{exportable_entity}/disable"
 *   }
 * )
 */
class ExportableEntity extends ConfigEntityBase implements ExportableEntityInterface {

  /**
   * Entity ID.
   *
   * @var string
   */
  protected $id;

  /**
   * Entity label.
   *
   * @var string
   */
  protected $label;

  /**
   * The id of the resolver plugin.
   *
   * @var string
   */
  protected $resolver;

  /**
   * The export format.
   *
   * @var string
   */
  protected $format;

  /**
   * A flag to tell if it should export other entities where an entity appears.
   *
   * @var bool
   */
  protected $export_referencing_entities = FALSE;

  /**
   * The recursion level.
   *
   * @var int
   */
  protected $recursion_level = 1;

  /**
   * A flag to tell whether it should export data when a CRUD operation happens
   * on CLI.
   *
   * @var bool
   */
  protected $export_when_crud_happens_on_cli = TRUE;

  /**
   * A flag to tell whether to request a build when a CRUD operation exports
   * data on CLI.
   *
   * @var bool
   */
  protected $request_build_when_crud_exports_on_cli = FALSE;

  /**
   * {@inheritdoc}
   */
  public function getEntityTypeIdString(): string {
    return explode('.', $this->id())[0];
  }

  /**
   * {@inheritdoc}
   */
  public function getBundle(): string {
    return explode('.', $this->id())[1];
  }

  /**
   * {@inheritdoc}
   */
  public function getResolver(): ?string {
    return $this->resolver;
  }

  /**
   * {@inheritdoc}
   */
  public function setResolver(string $resolver): void {
    $this->resolver = $resolver;
  }

  /**
   * {@inheritdoc}
   */
  public function getDirectory(): string {
    return EntityExporterInterface::BASE_DIR . '/' . $this->getEntityTypeIdString() . '/' . $this->getBundle() . EntityExporterInterface::OPTIONAL_SUB_DIR_TOKEN;
  }

  /**
   * {@inheritdoc}
   */
  public function getFilename(): string {
    return EntityExporterInterface::ENTITY_ID_TOKEN;
  }

  /**
   * {@inheritdoc}
   */
  public function getExtension(EntityInterface $entity): string {
    $definitions = $this->outputFormatterManager()->getDefinitions();
    return !empty($definitions[$this->format]) ? $definitions[$this->format]['extension'] : $this->format;
  }

  /**
   * {@inheritdoc}
   */
  public function outputFormatterManager(): OutputFormatterManagerInterface {
    return \Drupal::service('plugin.manager.static_output_formatter');
  }

  /**
   * {@inheritdoc}
   */
  public function getFormat(): ?string {
    return $this->format;
  }

  /**
   * {@inheritdoc}
   */
  public function setFormat(string $format): void {
    $this->format = $format;
  }

  /**
   * {@inheritdoc}
   */
  public function getExportReferencingEntities(): bool {
    return $this->export_referencing_entities;
  }

  /**
   * {@inheritdoc}
   */
  public function setExportReferencingEntities(bool $flag) :void {
    $this->export_referencing_entities = $flag;
  }

  /**
   * {@inheritdoc}
   */
  public function getRecursionLevel(): ?int {
    return $this->recursion_level;
  }

  /**
   * {@inheritdoc}
   */
  public function setRecursionLevel(int $recursionLevel): void {
    $this->recursion_level = $recursionLevel;
  }

  /**
   * {@inheritdoc}
   */
  public function getExportWhenCrudHappensOnCli(): bool {
    return $this->export_when_crud_happens_on_cli;
  }

  /**
   * {@inheritdoc}
   */
  public function setExportWhenCrudHappensOnCli(bool $flag) :void {
    $this->export_when_crud_happens_on_cli = $flag;
  }

  /**
   * {@inheritdoc}
   */
  public function getRequestBuildWhenCrudExportsOnCli(): bool {
    return $this->request_build_when_crud_exports_on_cli;
  }

  /**
   * {@inheritdoc}
   */
  public function setRequestBuildWhenCrudExportsOnCli(bool $flag) :void {
    $this->request_build_when_crud_exports_on_cli = $flag;
  }

}
