<?php

namespace Drupal\static_export\Entity;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\static_export\Plugin\OutputFormatterManagerInterface;

/**
 * Provides an interface defining an Exportable entity.
 */
interface ExportableEntityInterface extends ConfigEntityInterface {

  public const ENTITY_ID_TOKEN = '[entity:id]';

  /**
   * Get the entity type id used in this exportable entity.
   *
   * @return string
   *   Entity type id.
   */
  public function getEntityTypeIdString(): string;

  /**
   * Get the bundle used in this exportable entity.
   *
   * @return string
   *   Bundle.
   */
  public function getBundle(): string;

  /**
   * Get resolver.
   *
   * @return string
   *   The id of the resolver plugin.
   */
  public function getResolver(): ?string;

  /**
   * Set resolver.
   *
   * @param string $resolver
   *   The id of the resolver plugin.
   */
  public function setResolver(string $resolver): void;

  /**
   * Get export directory.
   *
   * It may return a string containing self::OPTIONAL_SUB_DIR_TOKEN. In such
   * cases, that token is replaced with the value of a subdirectory.
   *
   * @see ExpotableEntity::getDirectory
   *
   * @return string
   *   The directory where exported entity data is saved.
   */
  public function getDirectory(): string;

  /**
   * Get export filename.
   *
   * It must contain the token self::ENTITY_ID_TOKEN that is later replaced by
   * the entity id. Using a token ensures that filename contains the entity id,
   * offering flexibility at the same time.
   *
   * @return string
   *   The export filename containing an entity id token.
   */
  public function getFilename(): string;

  /**
   * Get export extension.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Entity to get extension from.
   *
   * @return string
   *   The export extension
   */
  public function getExtension(EntityInterface $entity): string;

  /**
   * Gets the output formatter manager.
   *
   * Since this is a special class for config entities (@ConfigEntityType),
   * dependencies can't be easily provided using Symfony's Dependency
   * Injection. Drupal Core uses a different strategy for such cases, where
   * dependencies are returned by a wrapper method like this one.
   *
   * @return \Drupal\static_export\Plugin\OutputFormatterManagerInterface
   */
  public function outputFormatterManager(): OutputFormatterManagerInterface;

  /**
   * Get export format.
   *
   * @return string|null
   *   The export format.
   */
  public function getFormat(): ?string;

  /**
   * Set export format.
   *
   * @param string $format
   */
  public function setFormat(string $format): void;

  /**
   * Tells whether it should export other entities where an entity appears.
   *
   * @return bool
   *   True or false.
   */
  public function getExportReferencingEntities(): bool;

  /**
   * Sets whether it should export other entities where an entity appears.
   *
   * @param bool $flag
   *   A boolean flag.
   */
  public function setExportReferencingEntities(bool $flag): void;

  /**
   * Get recursion level.
   *
   * @return int|null
   *   The recursion level, from 0 (infinite) to any positive number.
   */
  public function getRecursionLevel(): ?int;

  /**
   * Set recursion level.
   *
   * @param int $recursionLevel
   */
  public function setRecursionLevel(int $recursionLevel): void;

  /**
   * Tells whether it should export data when a CRUD operation happens on CLI.
   *
   * @return bool
   *   True or false.
   */
  public function getExportWhenCrudHappensOnCli(): bool;

  /**
   * Sets whether it should export data when a CRUD operation happens on CLI.
   *
   * @param bool $flag
   *   A boolean flag.
   */
  public function setExportWhenCrudHappensOnCli(bool $flag): void;

  /**
   * Tells whether to request a build when a CRUD operation exports data on CLI.
   *
   * @return bool
   *   True or false.
   */
  public function getRequestBuildWhenCrudExportsOnCli(): bool;

  /**
   * Sets whether to request a build when a CRUD operation exports data on CLI.
   *
   * @param bool $flag
   *   A boolean flag.
   */
  public function setRequestBuildWhenCrudExportsOnCli(bool $flag): void;

}
