<?php

namespace Drupal\static_export\EventSubscriber;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigCrudEvent;
use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ConfigRenameEvent;
use Drupal\static_export\Plugin\ExporterManagerInterface;
use Drupal\static_export\Service\MessengerService;
use Drupal\static_suite\Utility\StaticSuiteUtilsInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Throwable;

/**
 * Event subscriber for config entities.
 */
class ConfigEventSubscriber implements EventSubscriberInterface {

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Exporter Manager.
   *
   * @var \Drupal\static_export\Plugin\ExporterManagerInterface
   */
  protected $exporterManager;

  /**
   * Messenger service.
   *
   * @var \Drupal\static_export\Service\MessengerService
   */
  protected $messenger;

  /**
   * Static Suite Utils.
   *
   * @var \Drupal\static_suite\Utility\StaticSuiteUtilsInterface
   */
  protected $staticSuiteUtils;

  /**
   * The exporter that will handle the event.
   *
   * @var \Drupal\static_export\Plugin\ExporterInterface
   */
  protected $exporter;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config factory.
   * @param \Drupal\static_export\Plugin\ExporterManagerInterface $exporterManager
   *   Exporter Manager.
   * @param \Drupal\static_export\Service\MessengerService $messenger
   *   Messenger service.
   * @param \Drupal\static_suite\Utility\StaticSuiteUtilsInterface $staticSuiteUtils
   *   Static Suite Utils.
   */
  public function __construct(ConfigFactoryInterface $configFactory, ExporterManagerInterface $exporterManager, MessengerService $messenger, StaticSuiteUtilsInterface $staticSuiteUtils) {
    $this->configFactory = $configFactory;
    $this->exporterManager = $exporterManager;
    $this->messenger = $messenger;
    $this->staticSuiteUtils = $staticSuiteUtils;

  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[ConfigEvents::SAVE][] = ['configSave'];
    $events[ConfigEvents::DELETE][] = ['configDelete'];
    $events[ConfigEvents::RENAME][] = ['configRename'];
    return $events;
  }

  /**
   * Reacts to a save event.
   *
   * @param \Drupal\Core\Config\ConfigCrudEvent $event
   *   The configuration event.
   *
   * @return bool
   *   TRUE if everything is OK.
   */
  public function configSave(ConfigCrudEvent $event) {
    return $this->configCrudEventExport($event->getConfig(), 'export');
  }

  /**
   * Reacts to a delete event.
   *
   * @param \Drupal\Core\Config\ConfigCrudEvent $event
   *   The configuration event.
   *
   * @return bool
   *   TRUE if everything is OK.
   */
  public function configDelete(ConfigCrudEvent $event) {
    return $this->configCrudEventExport($event->getConfig(), 'delete');
  }

  /**
   * Reacts to a rename event.
   *
   * @param \Drupal\Core\Config\ConfigRenameEvent $event
   *   The configuration event.
   *
   * @return bool
   *   TRUE if everything is OK.
   */
  public function configRename(ConfigRenameEvent $event) {
    // Deleting the old configuration object is not yet supported.
    return $this->configCrudEventExport($event->getConfig(), 'export');
  }

  /**
   * Exports or deletes a config.
   *
   * @param \Drupal\Core\Config\Config $configObject
   *   The configuration object that caused the event to fire.
   * @param string $operation
   *   "export" or "delete".
   *
   * @return bool
   *   TRUE on success
   */
  protected function configCrudEventExport(Config $configObject, string $operation): bool {
    try {
      // This event is triggered with lots of CRUD operations. Reuse the same
      // exporter if this event is called more than once in the same thread.
      if (!$this->exporter) {
        $this->exporter = $this->exporterManager->getInstance(['plugin_id' => 'config']);
      }

      // This event is triggered with lots of CRUD operations, so we need to
      // filter it after calling the exporter. The exporter does the same,
      // checking its params, but it would throw an exception.
      if (!$this->exporter->isExportable(
        [$this->exporter::PARAM_CONFIG_NAME => $configObject->getName()]
      )) {
        return FALSE;
      }

      // If running on CLI, check if a export operation must be executed.
      if ($this->staticSuiteUtils->isRunningOnCli() && !$this->configFactory->get('static_export.settings')->get('exportable_config.export_when_crud_happens_on_cli')) {
        return FALSE;
      }

      $requestBuild = TRUE;
      // If running on CLI, check if a build must be requested.
      if ($this->staticSuiteUtils->isRunningOnCli()) {
        $requestBuild = !$this->configFactory->get('static_export.settings')->get('exportable_config.request_build_when_crud_exports_on_cli');
      }

      $this->exporter->setMustBuild($requestBuild);
      if ($operation === 'delete') {
        $fileCollectionGroup = $this->exporter->delete(
          [
            $this->exporter::PARAM_CONFIG_NAME => $configObject->getName(),
          ]
        );
      }
      else {
        $fileCollectionGroup = $this->exporter->export(
          [
            $this->exporter::PARAM_CONFIG_NAME => $configObject->getName(),
          ]
        );
      }
      $this->messenger->addFileCollectionGroup($fileCollectionGroup);
    }
    catch (Throwable $e) {
      $message = $e->getMessage();
      $this->messenger->addError($message);
      if ($this->exporter) {
        $this->exporter->logger()->error($message);
      }
    }
    return TRUE;
  }

}
