<?php

namespace Drupal\static_export\EventSubscriber;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\locale\LocaleEvent;
use Drupal\locale\LocaleEvents;
use Drupal\static_export\Plugin\ExporterManagerInterface;
use Drupal\static_export\Service\MessengerService;
use Drupal\static_suite\Utility\StaticSuiteUtilsInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Throwable;

/**
 * Event subscriber for translations.
 */
class TranslationEventSubscriber implements EventSubscriberInterface {

  /**
   * Drupal config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Exporter Manager.
   *
   * @var \Drupal\static_export\Plugin\ExporterManagerInterface
   */
  protected $exporterManager;

  /**
   * Messenger service.
   *
   * @var \Drupal\static_export\Service\MessengerService
   */
  protected $messenger;

  /**
   * Static Suite Utils.
   *
   * @var \Drupal\static_suite\Utility\StaticSuiteUtilsInterface
   */
  protected $staticSuiteUtils;

  /**
   * The exporter that will handle the event.
   *
   * @var \Drupal\static_export\Plugin\ExporterInterface
   */
  protected $exporter;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Drupal config factory.
   * @param \Drupal\static_export\Plugin\ExporterManagerInterface $exporterManager
   *   Exporter Manager.
   * @param \Drupal\static_export\Service\MessengerService $messenger
   *   Messenger service.
   * @param \Drupal\static_suite\Utility\StaticSuiteUtilsInterface $static_suite_utils
   *   Static Suite Utils.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ExporterManagerInterface $exporterManager, MessengerService $messenger, StaticSuiteUtilsInterface $static_suite_utils) {
    $this->configFactory = $config_factory;
    $this->exporterManager = $exporterManager;
    $this->messenger = $messenger;
    $this->staticSuiteUtils = $static_suite_utils;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[LocaleEvents::SAVE_TRANSLATION][] = ['onSaveTranslation'];
    return $events;
  }

  /**
   * Reacts to a save event.
   *
   * @param \Drupal\locale\LocaleEvent $event
   *   A locale save event.
   *
   * @return bool
   *   TRUE on success
   */
  public function onSaveTranslation(LocaleEvent $event): bool {
    // @todo - add support for exporting multiple files at once.
    $langCodes = $event->getLangCodes();

    // Multiple langcodes are possible. Export the first one as the master
    // language, and the other ones as slaves.
    $masterLangCode = array_shift($langCodes);
    // If running on CLI, check if a export operation must be executed.
    if ($this->staticSuiteUtils->isRunningOnCli() && !$this->configFactory->get('static_export.settings')->get('exportable_translation.export_when_crud_happens_on_cli')) {
      return FALSE;
    }

    $requestBuild = TRUE;
    // If running on CLI, check if a build must be requested.
    if ($this->staticSuiteUtils->isRunningOnCli()) {
      $requestBuild = !$this->configFactory->get('static_export.settings')->get('exportable_translation.request_build_when_crud_exports_on_cli');
    }

    // Export the master language.
    $exporter = NULL;
    try {
      $exporter = $this->exporterManager->getInstance(['plugin_id' => 'translation']);
      $exporter->setMustBuild($requestBuild);

      /*
      // Export remaining languages.
      // This doesn't work, because we need to use events to capture the master
      // export process and hook sub-exports inside. It can't be done outside
      // the master export. Wait until exporters are refactored to support
      // multiple files
      foreach ($langCodes as $langCode) {
      $slaveExporter = $this->exporterManager->createInstance('translation');
      $slaveExporter->makeSlaveOf($exporter;
      $slaveFileCollectionGroup = $slaveExporter->export([
      $slaveExporter::PARAM_LANG_CODE => $langCode,
      ]);
      $fileCollectionGroup->addFileCollections($slaveFileCollectionGroup->getFileCollections());
      }
       */

      $fileCollectionGroup = $exporter->export([
        $exporter::PARAM_LANG_CODE => $masterLangCode,
      ]);
      $this->messenger->addFileCollectionGroup($fileCollectionGroup);
    }
    catch (PluginException $e) {
      $message = $e->getMessage();
      $this->messenger->addError($message);
      if ($exporter) {
        $exporter->logger()->error($message);
      }
    }
    catch (Throwable $e) {
      $message = $e->getMessage();
      $this->messenger->addError($message);
      if ($exporter) {
        $exporter->logger()->error($message);
      }
    }

    return TRUE;
  }

}
