<?php

namespace Drupal\static_export\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\static_export\Entity\ExportableEntityManagerInterface;
use Drupal\static_export\Event\StaticExportEvents;
use Drupal\static_export\Event\StaticExportEvent;
use Drupal\static_export\File\FileCollection;
use Drupal\static_export\Plugin\ExporterInterface;
use Drupal\static_export\Plugin\ExporterManagerInterface;
use Drupal\static_export\Plugin\ResolverManagerInterface;
use Drupal\static_suite\Entity\EntityUtils;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriber for variants.
 */
class VariantEventSubscriber implements EventSubscriberInterface {

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Exporter Manager.
   *
   * @var \Drupal\static_export\Plugin\ExporterManagerInterface
   */
  protected $exporterManager;

  /**
   * Resolver manager.
   *
   * @var \Drupal\static_export\Plugin\ResolverManagerInterface
   */
  protected $resolverManager;

  /**
   * Exportable entity manager.
   *
   * @var \Drupal\static_export\Entity\ExportableEntityManagerInterface
   */
  protected $exportableEntityManager;

  /**
   * Entity Utils.
   *
   * @var \Drupal\static_suite\Entity\EntityUtils
   */
  protected $entityUtils;

  /**
   * Constructs the EntityEventSubscriber object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\static_export\Plugin\ExporterManagerInterface $exporterManager
   *   Exporter Manager.
   * @param \Drupal\static_export\Plugin\ResolverManagerInterface $resolverManager
   *   Resolver Manager.
   * @param \Drupal\static_export\Entity\ExportableEntityManagerInterface $exportableEntityManager
   *   Exportable entity manager.
   * @param \Drupal\static_suite\Entity\EntityUtils $entityUtils
   *   Utils for working with entities.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ExporterManagerInterface $exporterManager, ResolverManagerInterface $resolverManager, ExportableEntityManagerInterface $exportableEntityManager, EntityUtils $entityUtils) {
    $this->configFactory = $config_factory;
    $this->exporterManager = $exporterManager;
    $this->resolverManager = $resolverManager;
    $this->exportableEntityManager = $exportableEntityManager;
    $this->entityUtils = $entityUtils;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[StaticExportEvents::WRITE_STARTS][] = ['onWriteStarts', 100];
    return $events;
  }

  /**
   * Reacts to a StaticExportEvents::WRITE_STARTS event.
   *
   * @param \Drupal\static_export\Event\StaticExportEvent $event
   *   The Static Export event.
   *
   * @return \Drupal\static_export\Event\StaticExportEvent
   *   The processed event.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function onWriteStarts(StaticExportEvent $event) {
    $eventExporter = $event->getExporter();
    $exporterOptions = $eventExporter->getOptions();
    // Always export variants unless this is already a variant export.
    if (empty($exporterOptions['variant'])) {
      $variantKeys = $eventExporter->getVariantKeys();
      if (count($variantKeys) > 0) {
        $fileCollection = $this->exportVariants($eventExporter, $variantKeys);
        $event->getFileCollection()->merge($fileCollection);
      }
    }
    return $event;
  }

  /**
   * Exports entities referenced by the original one.
   *
   * @param \Drupal\static_export\Plugin\ExporterInterface $entityExporter
   *   The event entity exporter.
   *
   * @return \Drupal\static_export\File\FileCollection
   *   A file collection.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\static_suite\StaticSuiteUserException
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  protected function exportVariants(ExporterInterface $entityExporter, array $variantKeys): FileCollection {
    $fileCollection = new FileCollection($entityExporter->uniqueId());
    $entityBeingExported = $entityExporter->getExporterItem();
    foreach ($variantKeys as $variantKey) {
      // We need a new instance, so create a new one instead of getting it.
      $exporter = $this->exporterManager->createInstance('entity');
      $exporter->makeSlaveOf($entityExporter);
      if ($entityExporter->getOperation() === ExporterInterface::OPERATION_WRITE) {
        $variantsFileCollectionGroup = $exporter->export(
          [
            $exporter::PARAM_ENTITY => $entityBeingExported,
            'variant' => $variantKey,
          ],
          TRUE,
          $entityExporter->mustLogToFile(),
          $entityExporter->isLock()
        );
      }
      else {
        $variantsFileCollectionGroup = $exporter->delete(
          [
            $exporter::PARAM_ENTITY => $entityBeingExported,
            'variant' => $variantKey,
          ],
          TRUE,
          $entityExporter->mustLogToFile(),
          $entityExporter->isLock()
        );
      }

      $fileCollection->mergeMultiple($variantsFileCollectionGroup->getFileCollections());
    }
    return $fileCollection;
  }

}
