<?php

namespace Drupal\static_export\File;

/**
 * A class for holding a set of files.
 */
class FileCollection {

  /**
   * A unique Id to identify which process created this FileCollection.
   *
   * @var string
   */
  protected $uniqueId;

  /**
   * An array of FileItem.
   *
   * @var FileItem[]
   */
  protected $fileItems = [];

  /**
   * FileCollection constructor.
   *
   * @param string $uniqueId
   *   A unique id.
   */
  public function __construct(string $uniqueId) {
    $this->uniqueId = $uniqueId;
  }

  /**
   * Get unique id.
   *
   * @return string
   *   A unique id.
   */
  public function uniqueId(): string {
    return $this->uniqueId;
  }

  /**
   * Get all FileItems.
   *
   * @return \Drupal\static_export\File\FileItem[]
   *   An array of FileItem.
   */
  public function getFileItems(): array {
    return $this->fileItems;
  }

  /**
   * Get all executed FileItems.
   *
   * @return \Drupal\static_export\File\FileItem[]
   *   An array of FileItem.
   */
  public function getExecutedFileItems(): array {
    $executedItems = [];
    foreach ($this->fileItems as $fileItem) {
      if ($fileItem->isExecuted()) {
        $executedItems[] = $fileItem;
      }
    }
    return $executedItems;
  }

  /**
   * Get all executed FileItems paths, avoiding duplicates.
   *
   * @return string[]
   *   An array of file paths.
   */
  public function getExecutedFileItemPaths(): array {
    $executedFileItemPaths = [];
    $executedItems = $this->getExecutedFileItems();
    foreach ($executedItems as $item) {
      $executedFileItemPaths[$item->getFilePath()] = TRUE;
    }
    return array_keys($executedFileItemPaths);
  }

  /**
   * Set all FileItems.
   *
   * @param \Drupal\static_export\File\FileItem[] $fileItems
   *   An array of FileItem.
   */
  public function setFileItems(array $fileItems) {
    $this->fileItems = $fileItems;
  }

  /**
   * Get first FileItem.
   *
   * @return \Drupal\static_export\File\FileItem|null
   *   First FileItem.
   */
  public function getFirstFileItem(): ?FileItem {
    if (isset($this->fileItems[0])) {
      return $this->fileItems[0];
    }
    return NULL;
  }

  /**
   * Set an individual FileItem.
   *
   * @param \Drupal\static_export\File\FileItem $fileItem
   *   An individual FileItem.
   */
  public function addFileItem(FileItem $fileItem) {
    $this->fileItems[] = $fileItem;
  }

  /**
   * Returns the size of the collection.
   *
   * @return int
   *   The size of the collection.
   */
  public function size() {
    return count($this->fileItems);
  }

  /**
   * Tells if this FileCollection is empty.
   *
   * @return bool
   *   True if empty
   */
  public function isEmpty() {
    return $this->size() == 0 ? TRUE : FALSE;
  }

  /**
   * Merges a FileCollection with this one.
   *
   * @param FileCollection $anotherFileCollection
   *   Another FileCollection.
   */
  public function merge(FileCollection $anotherFileCollection) {
    $otherItems = $anotherFileCollection->getFileItems();
    foreach ($otherItems as $otherItem) {
      $this->addFileItem($otherItem);
    }
  }

  /**
   * Merges an array of fileCollections with this one.
   *
   * @param FileCollection[] $anotherFileCollections
   *   An array of another fileCollections.
   */
  public function mergeMultiple(array $anotherFileCollections) {
    foreach ($anotherFileCollections as $anotherFileCollection) {
      $this->merge($anotherFileCollection);
    }
  }

  /**
   * Check if any file item has been executed (saved to the data-dir).
   *
   * @return bool
   *   True if any of them has been executed.
   */
  public function isAnyFileItemExecuted() {
    foreach ($this->fileItems as $item) {
      if ($item->isExecuted()) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Check if this FileCollection contains data to be used in a preview build.
   *
   * @return bool
   *   True if it contains data for a preview.
   */
  public function isPreview() {
    if ($this->size() === 1) {
      $fileItem = $this->getFirstFileItem();
      if ($fileItem && $fileItem->getCurrentStatus() === 0 && $fileItem->getOldStatus() === 0) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
