<?php

namespace Drupal\static_export\File;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;

/**
 * A class for formatting a FileCollection.
 */
class FileCollectionFormatter {

  /**
   * A FileCollection to format.
   *
   * @var \Drupal\static_export\File\FileCollection
   */
  protected $fileCollection;

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * FileCollectionFormatter constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The base config provider.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * Set a FileCollection.
   *
   * @param \Drupal\static_export\File\FileCollection $fileCollection
   *   A FileCollection to format.
   */
  public function setFileCollection(FileCollection $fileCollection) {
    $this->fileCollection = $fileCollection;
  }

  /**
   * Returns a default string representation (in text format)
   *
   * @return string
   *   A textual representation.
   */
  public function __toString() {
    return implode("\n", $this->getTextLines());
  }

  /**
   * Returns a text representation.
   *
   * @param int $delta
   *   Batch delta, useful only in a batch operation.
   * @param int $total
   *   Total size of batch, useful only in a batch operation.
   *
   * @return string
   *   A textual representation.
   */
  public function toText(int $delta = 0, int $total = 0) {
    return implode("\n", $this->getTextLines($delta, $total));
  }

  /**
   * Returns a html representation.
   *
   * This method is filtered by user, because permissions apply.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $user
   *   The user to get the html for.
   * @param int $delta
   *   Batch delta, useful only in a batch operation.
   * @param int $total
   *   Total size of batch, useful only in a batch operation.
   *
   * @return string
   *   A html representation.
   */
  public function toHtml(AccountProxyInterface $user, int $delta = 0, int $total = 0) {
    return "<ul><li>" . implode("<li>", $this->getHtmlLines($user, $delta, $total)) . "</li></ul>";
  }

  /**
   * Formats a FileCollection using plain text format and returns an array.
   *
   * @param int $delta
   *   Batch delta, useful only in a batch operation.
   * @param int $total
   *   Total size of batch, useful only in a batch operation.
   *
   * @return array
   *   An array of formatted text lines.
   */
  public function getTextLines(int $delta = 0, int $total = 0) {
    $lines = [];
    foreach ($this->fileCollection->getFileItems() as $index => $fileItem) {
      if ($total == 0) {
        if ($index == 0) {
          $line = sprintf("%2s", "");
        }
        else {
          $line = sprintf("%2s ", json_decode('"\u21B3"'));
        }
      }
      else {
        if ($index == 0) {
          $line = sprintf("%11s ", "[$delta/$total]");
        }
        else {
          $line = sprintf("%13s ", json_decode('"\u21B3"'));
        }
      }
      $line .= sprintf("[%6s] ", $fileItem->getBenchmark() . "s");
      $line .= "[" . $this->fileCollection->uniqueId() . "] ";
      $runMode = $this->fileCollection->isPreview() ? 'preview' : 'live';
      $line .= "[" . $runMode . "] ";
      if ($fileItem->isExecuted() === NULL) {
        $executed = 'unknown';
      }
      else {
        $executed = $fileItem->isExecuted() ? 'executed' : 'unneeded';
      }
      $line .= "[" . $fileItem->getOperation() . " $executed] ";
      $line .= "[ID: " . $fileItem->getId() . "] ";
      $line .= $fileItem->getLabel() . " | ";
      $line .= $fileItem->getFilePath();
      $lines[] = $line;
    }
    return $lines;
  }

  /**
   * Formats a FileCollection using html format and returns an array.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $user
   *   The user to get the html for.
   * @param int $delta
   *   Batch delta.
   * @param int $total
   *   Total size of batch.
   *
   * @return array
   *   An array of formatted html lines.
   */
  public function getHtmlLines(AccountProxyInterface $user, int $delta = 0, int $total = 0) {
    $lines = $this->getTextLines($delta, $total);
    if ($user->hasPermission("view static export files")) {
      $fileViewerUrl = Url::fromRoute('static_export.file_viewer')->toString();
      foreach ($lines as $index => $line) {
        // Parse unique id link.
        if (preg_match("/ \[(\d{4}-\d{2}-\d{2}_\d{2}-\d{2}-\d{2}.\d{6}__\d{4})\] /", $line, $matches)) {
          $htmlForFilePath = '<a href="' . $fileViewerUrl . '?path=' . $this->configFactory->get('static_export.settings')->get('log_dir') . '/' . $matches[1] . '.log" target="_blank">' . $matches[1] . '</a>';
          $lines[$index] = str_replace($matches[1], $htmlForFilePath, $lines[$index]);
        }

        // Parse file link.
        if (preg_match("/ (\S+)$/", $line, $matches)) {
          $htmlForFilePath = '<a href="' . $fileViewerUrl . '?path=' . $matches[1] . '" target="_blank">' . $matches[1] . '</a>';
          $lines[$index] = str_replace($matches[1], $htmlForFilePath, $lines[$index]);
        }
      }
    }
    return $lines;
  }

}
