<?php

namespace Drupal\static_export\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\static_export\Plugin\ConfigExporterInterface;
use Drupal\static_export\Plugin\OutputFormatterManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form handler for exportable configurations.
 */
class ExportableConfigSettingsForm extends ConfigFormBase {

  /**
   * The static output formatter manager.
   *
   * @var \Drupal\static_export\Plugin\OutputFormatterManagerInterface
   */
  protected $outputFormatterManager;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\static_export\Plugin\OutputFormatterManagerInterface $outputFormatterManager
   *   The static output formatter manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, OutputFormatterManagerInterface $outputFormatterManager) {
    ConfigFormBase::__construct($config_factory);
    $this->outputFormatterManager = $outputFormatterManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.static_output_formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_export_exportable_config_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['static_export.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');

    $formatDefinitions = $this->outputFormatterManager->getDefinitions();
    $formatOptions = [];
    foreach ($formatDefinitions as $formatterDefinition) {
      $formatOptions[$formatterDefinition['id']] = $formatterDefinition['label'];
    }
    $form['format'] = [
      '#type' => 'select',
      '#title' => $this->t('Export format'),
      '#options' => $formatOptions,
      '#default_value' => $config->get('exportable_config.format'),
      '#prefix' => $this->t('All exportable configurations are exported to the same directory (%dir), using the same export format. File name is derived from each exported configuration object.', ['%dir' => $this->configFactory->get('static_export.settings')->get('data_dir') . ConfigExporterInterface::BASE_DIR]),
      '#description' => $this->t('Format for the exported data.'),
      '#required' => TRUE,
    ];

    $form['cli'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('CLI options'),
      '#description' => $this->t('Options for CRUD operations that happens outside a web server (e.g.- a cron or a drush command editing or adding entities, etc)'),
    ];

    $form['cli']['export_when_crud_happens_on_cli'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Export configurations when a CRUD operation happens on CLI (not recommended)'),
      '#default_value' => $config->get('exportable_config.export_when_crud_happens_on_cli'),
      '#description' => $this->t('Contrary to exportable entities and translations, exporting configurations inside a non-interactive process could lead to problems. For example, running a "drush cim" command would trigger export operations that could be broken due to some configuration not being already updated. Therefore, it is recommended to keep this option disabled and to manually export configuration objects after "drush cim" has finished.'),
      '#required' => FALSE,
    ];

    $form['cli']['request_build_when_crud_exports_on_cli'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Request a build when a CRUD operation exports configurations on CLI (not recommended).'),
      '#default_value' => $config->get('exportable_config.request_build_when_crud_exports_on_cli'),
      '#description' => $this->t('It is not recommended to request a build from a non-interactive process, because it will lead to several builds taking place in an uncontrolled way.'),
      '#required' => FALSE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');
    $config
      ->set('exportable_config.format', $form_state->getValue('format'))
      ->set('exportable_config.export_when_crud_happens_on_cli', $form_state->getValue('export_when_crud_happens_on_cli'))
      ->set('exportable_config.request_build_when_crud_exports_on_cli', $form_state->getValue('request_build_when_crud_exports_on_cli'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
