<?php

namespace Drupal\static_export\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\static_export\Plugin\OutputFormatterManagerInterface;
use Drupal\static_export\Plugin\TranslationExporterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form handler for exportable translation.
 */
class ExportableTranslationSettingsForm extends ConfigFormBase {

  /**
   * The static output formatter manager.
   *
   * @var \Drupal\static_export\Plugin\OutputFormatterManagerInterface
   */
  protected $outputFormatterManager;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\static_export\Plugin\OutputFormatterManagerInterface $outputFormatterManager
   *   The static output formatter manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, OutputFormatterManagerInterface $outputFormatterManager) {
    ConfigFormBase::__construct($config_factory);
    $this->outputFormatterManager = $outputFormatterManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.static_output_formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_export_exportable_translation_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['static_export.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');

    $formatDefinitions = $this->outputFormatterManager->getDefinitions();
    $formatOptions = [];
    foreach ($formatDefinitions as $formatterDefinition) {
      $formatOptions[$formatterDefinition['id']] = $formatterDefinition['label'];
    }
    $form['format'] = [
      '#type' => 'select',
      '#title' => $this->t('Export format'),
      '#options' => $formatOptions,
      '#default_value' => $config->get('exportable_translation.format'),
      '#prefix' => $this->t('All available translations are automatically exported to the same directory (%dir), using the same export format. File name is derived from each language code (<em>"translation.{LANG_CODE}.{EXT}"</em>).', ['%dir' => $this->configFactory->get('static_export.settings')->get('data_dir') . TranslationExporterInterface::BASE_DIR]),
      '#description' => $this->t('Format for the exported data.'),
      '#required' => TRUE,
    ];

    $form['cli']['export_when_crud_happens_on_cli'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Export translations when a CRUD operation happens on CLI (not recommended)'),
      '#default_value' => $config->get('exportable_translation.export_when_crud_happens_on_cli'),
      '#description' => $this->t('It is recommended to export translations from these CRUD operation so there is no mismatch between Drupal database and exported data.'),
      '#required' => FALSE,
    ];

    $form['cli']['request_build_when_crud_exports_on_cli'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Request a build when a CRUD operation exports translations on CLI (recommended).'),
      '#default_value' => $config->get('exportable_translation.request_build_when_crud_exports_on_cli'),
      '#description' => $this->t('It is not recommended to request a build from a non-interactive process, because it will lead to several builds taking place in an uncontrolled way.'),
      '#required' => FALSE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('static_export.settings');
    $config
      ->set('exportable_translation.format', $form_state->getValue('format'))
      ->set('exportable_translation.export_when_crud_happens_on_cli', $form_state->getValue('export_when_crud_happens_on_cli'))
      ->set('exportable_translation.request_build_when_crud_exports_on_cli', $form_state->getValue('request_build_when_crud_exports_on_cli'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
