<?php

namespace Drupal\static_export\Plugin\Exporter;

use Drupal\static_export\Plugin\ConfigExporterInterface;
use Drupal\static_export\Plugin\ExporterOutputConfig;
use Drupal\static_export\Plugin\ExporterBase;
use Drupal\static_suite\StaticSuiteUserException;

/**
 * Exporter for Drupal configuration objects.
 *
 * @StaticExporter(
 *  id = "config",
 *  label = @Translation("Configuration exporter"),
 *  description = @Translation("Exports configuration entities to the filesystem"),
 * )
 */
class ConfigExporter extends ExporterBase implements ConfigExporterInterface {

  public const PARAM_CONFIG_NAME = 'config-name';

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\Core\Config\Config
   *   Drupal configuration object.
   */
  public function getExporterItem() {
    if (empty($this->options[self::PARAM_CONFIG_NAME])) {
      return NULL;
    }

    if (!$this->exporterItem[$this->options[self::PARAM_CONFIG_NAME]]) {
      $this->exporterItem[$this->options[self::PARAM_CONFIG_NAME]] = $this->configFactory->get(
        $this->options[self::PARAM_CONFIG_NAME]
      );
    }

    return $this->exporterItem[$this->options[self::PARAM_CONFIG_NAME]];
  }

  /**
   * {@inheritdoc}
   */
  public function getExporterItemId() {
    return $this->getExporterItem()->getName();
  }

  /**
   * {@inheritdoc}
   */
  public function getExporterItemLabel() {
    return $this->getExporterItem()->getName();
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public function checkParams(array $options, bool $isStandalone = FALSE) {
    if (!isset($options[self::PARAM_CONFIG_NAME])) {
      throw new StaticSuiteUserException("Param '" . self::PARAM_CONFIG_NAME . "' is not defined.");
    }

    // Do not check if self::PARAM_CONFIG_NAME is exportable, since that is
    // done in $this->isExportable(), which is automatically called.
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  protected function getOutputDefinition(): ExporterOutputConfig {
    $config = $this->configFactory->get('static_export.settings');
    $dir = $config->get('data_dir') . self::BASE_DIR;
    $filename = $this->options[self::PARAM_CONFIG_NAME];
    $format = $config->get('exportable_config.format');

    // Load the OutputFormatter plugin definition to get its extension.
    $definitions = $this->outputFormatterManager->getDefinitions();
    $extension = !empty($definitions[$format]) ? $definitions[$format]['extension'] : $format;

    return new ExporterOutputConfig($dir, $filename, $extension, $format);
  }

  /**
   * {@inheritdoc}
   */
  protected function calculateDataFromResolver() {
    return $this->getExporterItem()->getRawData();
  }

  /**
   * Get exportable config names.
   *
   * @return array
   *   Array with exportable config names.
   */
  protected function getExportableConfigNames(): array {
    return $this->configFactory->get('static_export.settings')->get('exportable_config.objects_to_export');
  }

  /**
   * {@inheritdoc}
   *
   * @param string $item
   *   Config name.
   */
  public function isExportable(array $options): bool {
    $exportableConfigNames = $this->getExportableConfigNames();
    if (in_array($options[self::PARAM_CONFIG_NAME], $exportableConfigNames, TRUE)) {
      return TRUE;
    }
    return FALSE;
  }

}
