<?php

namespace Drupal\static_export\Plugin\Exporter;

use Drupal\static_export\Plugin\EntityExporterInterface;
use Drupal\static_export\Plugin\ResolverInterface;
use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Entity\EntityInterface;
use Drupal\static_export\Entity\ExportableEntityManagerInterface;
use Drupal\static_export\Plugin\ExporterOutputConfig;
use Drupal\static_export\Plugin\ExporterBase;
use Drupal\static_export\Plugin\ResolverManagerInterface;
use Drupal\static_suite\StaticSuiteUserException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an entity exporter.
 *
 * @StaticExporter(
 *  id = "entity",
 *  label = @Translation("Entity exporter"),
 *  description = @Translation("Exports entities to the filesystem"),
 * )
 */
class EntityExporter extends ExporterBase implements EntityExporterInterface {

  public const PARAM_ENTITY_TYPE_ID = "entity-type-id";
  // Public const PARAM_ENTITY_BUNDLE_ID = "entity-bundle-id";.
  public const PARAM_ENTITY_ID = "entity-id";
  public const PARAM_ENTITY = "entity";

  /**
   * Resolver manager.
   *
   * @var \Drupal\static_export\Plugin\ResolverManagerInterface
   */
  protected $resolverManager;

  /**
   * Exportable entity manager.
   *
   * @var \Drupal\static_export\Entity\ExportableEntityManagerInterface
   */
  protected $exportableEntityManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->setExtraDependencies(
      $container->get("plugin.manager.static_data_resolver"),
      $container->get("static_export.exportable_entity_manager")
    );
    return $instance;
  }

  /**
   * Set extra dependencies for this exporter.
   *
   * @param \Drupal\static_export\Plugin\ResolverManagerInterface $resolverManager
   *   Resolver manager.
   * @param \Drupal\static_export\Entity\ExportableEntityManagerInterface $exportableEntityManager
   *   Exportable entity manager.
   */
  public function setExtraDependencies(ResolverManagerInterface $resolverManager, ExportableEntityManagerInterface $exportableEntityManager): void {
    $this->resolverManager = $resolverManager;
    $this->exportableEntityManager = $exportableEntityManager;
  }

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   Item being exported (an entity)
   */
  public function getExporterItem() {
    return $this->options[self::PARAM_ENTITY];
  }

  /**
   * {@inheritdoc}
   */
  public function getExporterItemId() {
    return $this->getExporterItem()->id();
  }

  /**
   * {@inheritdoc}
   */
  public function getExporterItemLabel() {
    return $this->getExporterItem()->label();
  }

  /**
   * Preprocess options.
   *
   * This exporter always needs an entity (at this moment), so its loaded
   * here if no provided by params. When everything is loaded thru GraphQL,
   * we won't need it because GraphQL will be able to provide everything
   * we need.
   *
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the storage handler couldn't be loaded.
   */
  public function preProcessOptions(array $options) {
    $options = parent::preProcessOptions($options);

    // Load an entity if enough options are provided.
    if (empty($options[self::PARAM_ENTITY]) && !empty($options[self::PARAM_ENTITY_TYPE_ID]) && !empty($options[self::PARAM_ENTITY_ID])) {
      // Load and set the entity.
      $options[self::PARAM_ENTITY] = $this->entityUtils->loadEntity(
        $options[self::PARAM_ENTITY_TYPE_ID],
        $options[self::PARAM_ENTITY_ID]
      );

      // Unset non necessary options because we have an entity.
      unset($options[self::PARAM_ENTITY_TYPE_ID],
        // $options[self::PARAM_ENTITY_BUNDLE_ID],
        $options[self::PARAM_ENTITY_ID]);
    }

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function checkParams(array $options, bool $isStandalone = FALSE) {
    // Check entity.
    if (empty($options[self::PARAM_ENTITY]) || !($options[self::PARAM_ENTITY] instanceof EntityInterface)) {
      throw new StaticSuiteUserException("Param '" . self::PARAM_ENTITY . "' must be an instance of Entity. Please, provide it directly, or provide the following parameters: '" . self::PARAM_ENTITY_TYPE_ID . "' and '" . self::PARAM_ENTITY_ID . "'");
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   *
   * There could be millions of entities exported to the same directory, so
   * we split them up into multiple directories, based on the entity id:
   *  - IDs from 0 to 9999: stored at the root level. Useful for small sites,
   *    where an extra subdirectory makes no sense
   *  - IDs from 10000 to 19999: stored into a "/10000" directory
   *  - IDs from 20000 to 29999: stored into a "/20000" directory
   * and so on...
   */
  protected function getOutputDefinition(): ?ExporterOutputConfig {
    $entity = $this->options[self::PARAM_ENTITY];
    $exportableEntity = $this->exportableEntityManager->getExportableEntity($entity);
    if ($exportableEntity && $exportableEntity->status()) {
      $dir = $this->configFactory->get('static_export.settings')->get('data_dir') . $exportableEntity->getDirectory();
      $entityId = $entity->id();
      if (is_numeric($entityId) && $entityId >= self::FILES_PER_DIR) {
        $thousands = (int) floor($entityId / self::FILES_PER_DIR) * self::FILES_PER_DIR;
        $dir = str_replace(self::OPTIONAL_SUB_DIR_TOKEN, '/' . $thousands, $dir);
      }
      $filename = str_replace(self::ENTITY_ID_TOKEN, $entityId, $exportableEntity->getFilename());
      $format = $exportableEntity->getFormat();
      $definitions = $this->outputFormatterManager->getDefinitions();
      $extension = !empty($definitions[$format]) ? $definitions[$format]['extension'] : $format;

      return new ExporterOutputConfig($dir, $filename, $extension, $format);
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  protected function calculateDataFromResolver() {
    $resolver = $this->getResolver();
    if ($resolver) {
      $entity = $this->options[self::PARAM_ENTITY];
      $variant = $this->options['variant'] ?? NULL;
      $dataFromResolver = $resolver->resolve($entity, $variant);
      $definition = $resolver->getPluginDefinition();
      if (empty($definition['format']) && !is_array($dataFromResolver)) {
        throw new StaticSuiteUserException("Resolver '" . $resolver->getPluginId() . "' must return an array");
      }
      return $dataFromResolver;
    }
    return NULL;
  }

  /**
   * Get the resolver plugin being used by this exporter.
   *
   * @return \Drupal\static_export\Plugin\ResolverInterface
   *   Resolver plugin
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  protected function getResolver(): ?ResolverInterface {
    $entity = $this->options[self::PARAM_ENTITY];
    $exportableEntity = $this->exportableEntityManager->getExportableEntity($entity);
    if ($exportableEntity && $exportableEntity->status()) {
      $resolverId = $exportableEntity->getResolver();
      try {
        $resolver = $this->resolverManager->getInstance(['plugin_id' => $resolverId]);
      }
      catch (PluginException $e) {
        throw new StaticSuiteUserException("Unknown resolver: " . $resolverId);
      }
      return $resolver;
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function isExportable(array $options): bool {
    $entity = $options[self::PARAM_ENTITY];
    if (empty($entity) || !($entity instanceof EntityInterface)) {
      return FALSE;
    }

    // Check if there is an exportable entity for this $entity, and that it's
    // enabled.
    return $this->exportableEntityManager->isExportable($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getVariantKeys(): array {
    $resolver = $this->getResolver();
    if ($resolver) {
      $entity = $this->options[self::PARAM_ENTITY];
      return $resolver->getVariantKeys($entity);
    }
    return [];
  }

}
