<?php

namespace Drupal\static_export\Plugin\Exporter;

use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\locale\StringStorageInterface;
use Drupal\static_export\Plugin\ExporterOutputConfig;
use Drupal\static_export\Plugin\ExporterBase;
use Drupal\static_export\Plugin\TranslationExporterInterface;
use Drupal\static_suite\StaticSuiteUserException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Exporter for language translations.
 *
 * @StaticExporter(
 *  id = "translation",
 *  label = @Translation("Translation exporter"),
 *  description = @Translation("Exports translations to the filesystem"),
 * )
 */
class TranslationExporter extends ExporterBase implements TranslationExporterInterface {

  const PARAM_LANG_CODE = 'lang-code';

  /**
   * Locale storage.
   *
   * @var \Drupal\locale\StringStorageInterface
   */
  protected $localeStorage;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->setExtraDependencies(
      $container->get("locale.storage"),
      $container->get("language_manager")
    );
    return $instance;
  }

  /**
   * Set extra dependencies for this exporter.
   *
   * @param \Drupal\locale\StringStorageInterface $localeStorage
   *   Local storage.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   Language manager.
   */
  public function setExtraDependencies(StringStorageInterface $localeStorage, LanguageManagerInterface $languageManager): void {
    $this->localeStorage = $localeStorage;
    $this->languageManager = $languageManager;
  }

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\locale\StringInterface[]
   *   Array of StringInterface
   */
  public function getExporterItem() {
    if (empty($this->options[self::PARAM_LANG_CODE])) {
      return NULL;
    }

    if (!$this->exporterItem[$this->options[self::PARAM_LANG_CODE]]) {
      $this->exporterItem[$this->options[self::PARAM_LANG_CODE]] = $this->localeStorage->getTranslations([
        'language' => $this->options[self::PARAM_LANG_CODE],
        'translated' => TRUE,
      ]);
    }

    return $this->exporterItem[$this->options[self::PARAM_LANG_CODE]];
  }

  /**
   * {@inheritdoc}
   */
  public function getExporterItemId() {
    return 'translation_strings';
  }

  /**
   * {@inheritdoc}
   */
  public function getExporterItemLabel() {
    return 'Translation Strings ' . $this->options[self::PARAM_LANG_CODE];
  }

  /**
   * {@inheritdoc}
   */
  public function checkParams(array $options, bool $isStandalone = FALSE) {
    if (!isset($options[self::PARAM_LANG_CODE])) {
      throw new StaticSuiteUserException("Param '" . self::PARAM_LANG_CODE . "' is not defined.");
    }

    $langCodes = $this->languageManager->getLanguages();
    if (empty($langCodes[$options[self::PARAM_LANG_CODE]])) {
      throw new StaticSuiteUserException("Language '" . $options[self::PARAM_LANG_CODE] . "' is not enabled on this site.");
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  protected function getOutputDefinition(): ExporterOutputConfig {
    $config = $this->configFactory->get('static_export.settings');
    $dir = $config->get('data_dir') . self::BASE_DIR;
    $filename = 'translation.' . $this->options[self::PARAM_LANG_CODE];
    $format = $config->get('exportable_translation.format');

    // Load the OutputFormatter plugin definition to get its extension.
    $definitions = $this->outputFormatterManager->getDefinitions();
    $extension = !empty($definitions[$format]) ? $definitions[$format]['extension'] : $format;

    return new ExporterOutputConfig($dir, $filename, $extension, $format);
  }

  /**
   * {@inheritdoc}
   */
  protected function calculateDataFromResolver() {
    $translationData = [];
    foreach ($this->getExporterItem() as $translation) {
      $translationData[$translation->source] = $translation->translation;
    }

    return $translationData;
  }

}
