<?php

namespace Drupal\static_export\Plugin;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\static_suite\Entity\EntityUtils;
use Drupal\static_export\Event\StaticExportEvents;
use Drupal\static_suite\StaticSuiteException;
use Drupal\static_suite\Utility\UniqueIdHelperInterface;
use Drupal\static_export\File\FileCollection;
use Drupal\static_export\File\FileCollectionGroup;
use Drupal\static_export\File\FileCollectionWriter;
use Drupal\static_export\File\FileItem;
use Drupal\static_export\Event\StaticExportEvent;
use Drupal\static_suite\StaticSuiteUserException;
use Drupal\static_suite\Utility\StaticSuiteUtilsInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Throwable;

/**
 * A service for handling export of data.
 */
abstract class ExporterBase extends PluginBase implements ExporterInterface {

  use LoggerChannelTrait;

  /**
   * Type of operation being done.
   *
   * One of ExporterInterface::OPERATION_*.
   *
   * @var string
   */
  protected $operation;

  /**
   * Language code.
   *
   * @var string
   */
  protected $languageCode;

  /**
   * Event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcher
   */
  protected $eventDispatcher;

  /**
   * Output formatter manager.
   *
   * @var \Drupal\static_export\Plugin\OutputFormatterManagerInterface
   */
  protected $outputFormatterManager;

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Disk writer.
   *
   * @var \Drupal\static_export\File\FileCollectionWriter
   */
  protected $fileCollectionWriter;

  /**
   * Static Export utils.
   *
   * @var \Drupal\static_suite\Utility\StaticSuiteUtilsInterface
   */
  protected $staticSuiteUtils;

  /**
   * Unique ID helper.
   *
   * @var \Drupal\static_suite\Utility\UniqueIdHelperInterface
   */
  protected $uniqueIdHelper;

  /**
   * Final FileCollectionGroup the exporter returns after doing all its work.
   *
   * @var \Drupal\static_export\File\FileCollectionGroup
   */
  protected $resultingFileCollectionGroup;

  /**
   * Flag for knowing if this is a master export (the first one)
   *
   * @var bool
   */
  protected $isMasterExport = TRUE;

  /**
   * Options array.
   *
   * @var array
   */
  protected $options;

  /**
   * Standalone mode flag.
   *
   * @var bool
   */
  protected $isStandalone = FALSE;

  /**
   * Log To File flag.
   *
   * @var bool
   */
  protected $logToFile = TRUE;

  /**
   * Lock flag.
   *
   * @var bool
   */
  protected $lock = TRUE;

  /**
   * Build flag.
   *
   * @var bool
   */
  protected $mustBuild = FALSE;

  /**
   * Global config data.
   *
   * @var array
   */
  protected $globalConfig;

  /**
   * Output config data.
   *
   * @var \Drupal\static_export\Plugin\ExporterOutputConfig
   */
  protected $outputConfig;

  /**
   * Time start.
   *
   * @var float
   */
  protected $timeStart;

  /**
   * Time end.
   *
   * @var float
   */
  protected $timeEnd;

  /**
   * Unique identifier.
   *
   * @var string
   */
  protected $uniqueId;

  /**
   * Flag to indicate that this exporter should always write.
   *
   * @var bool
   */
  protected $isForceWrite = FALSE;

  /**
   * Data set by handle Resolver.
   *
   * @var mixed
   */
  protected $dataFromResolver;

  /**
   * Item (entity, config object, or any data) that is being exported.
   *
   * @var mixed
   */
  protected $exporterItem;

  /**
   * Entity Utils.
   *
   * @var \Drupal\static_suite\Entity\EntityUtils
   */
  protected $entityUtils;

  /**
   * Constructor for exporter.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher service.
   * @param \Drupal\static_export\Plugin\OutputFormatterManagerInterface $outputFormatterManager
   *   Output formatter manager.
   * @param \Drupal\static_export\File\FileCollectionWriter $file_collection_writer
   *   Disk Writer.
   * @param \Drupal\static_suite\Entity\EntityUtils $entity_utils
   *   Entity utils service.
   * @param \Drupal\static_suite\Utility\StaticSuiteUtilsInterface $static_suite_utils
   *   Static Export utils.
   * @param \Drupal\static_suite\Utility\UniqueIdHelperInterface $unique_id_helper
   *   Unique ID helper.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config_factory,
    EventDispatcherInterface $event_dispatcher,
    OutputFormatterManagerInterface $outputFormatterManager,
    FileCollectionWriter $file_collection_writer,
    EntityUtils $entity_utils,
    StaticSuiteUtilsInterface $static_suite_utils,
    UniqueIdHelperInterface $unique_id_helper
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
    $this->eventDispatcher = $event_dispatcher;
    $this->outputFormatterManager = $outputFormatterManager;
    $this->fileCollectionWriter = $file_collection_writer;
    $this->entityUtils = $entity_utils;
    $this->staticSuiteUtils = $static_suite_utils;
    $this->uniqueIdHelper = $unique_id_helper;
  }

  /**
   * {@inheritdoc}
   *
   * If a exporter needs extra dependencies not defined in this method, you can
   * call a local helper method
   * (e.g.- setExtraDependencies(Type1 $dep1, Type2 $dep2)) before returning
   * the newly created instance.
   *
   * @see \Drupal\static_export\Plugin\Exporter\TranslationExporter for an
   * example
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get("config.factory"),
      $container->get("event_dispatcher"),
      $container->get("plugin.manager.static_output_formatter"),
      $container->get("static_export.file_collection_writer"),
      $container->get("static_suite.entity_utils"),
      $container->get("static_suite.utils"),
      $container->get("static_export.unique_id_helper")
    );
  }

  /**
   * Get unique id.
   *
   * @return string
   *   The unique id.
   */
  public function uniqueId() {
    return $this->uniqueId;
  }

  /**
   * Set unique id.
   *
   * Should be used with caution.
   *
   * @param string $uniqueId
   *   A unique id.
   */
  public function setUniqueId(string $uniqueId) {
    $this->uniqueId = $uniqueId;
  }

  /**
   * Sets whether this export is a master export.
   *
   * @param bool $value
   *   True or false.
   */
  public function setIsMasterExport(bool $value) {
    $this->isMasterExport = $value;
  }

  /**
   * Gets whether this export is a master export.
   *
   * @return bool
   *   True or false.
   */
  public function isMasterExport() {
    return $this->isMasterExport;
  }

  /**
   * Preprocess options array.
   *
   * Useful when you need to load some kind of entity
   * based on the values of $options, and/or to reorganize
   * those $options.
   *
   * Not all exporters may need this, hence it's not defined on the
   * interface but on this base class.
   *
   * @param array $options
   *   Options for export operation.
   *
   * @return array
   *   Processed $options.
   */
  protected function preProcessOptions(array $options) {
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  abstract public function checkParams(array $options, bool $isStandalone = FALSE);

  /**
   * {@inheritdoc}
   */
  public function setOptions(array $options): void {
    $this->logMessage("Setting \$options = " . $this->optionsForLogging($options));
    $this->options = $options;
  }

  /**
   * {@inheritdoc}
   */
  public function getOptions(): array {
    return $this->options;
  }

  /**
   * {@inheritdoc}
   */
  public function getOperation(): string {
    return $this->operation;
  }

  /**
   * Set exporter standalone mode.
   *
   * This sets a flag that event subscribers should honor to avoid
   * generation of sub fileCollectionGroups. This is useful when an export
   * operation would trigger other sub exports and you don't want them.
   *
   * This flag is useless if event subscribers don't honor it.
   *
   * @param bool $isStandalone
   *   Standalone flag.
   */
  protected function setIsStandalone(bool $isStandalone) {
    $this->logMessage("Setting \$isStandalone = " . ($isStandalone ? "TRUE" : "FALSE"));
    $this->isStandalone = $isStandalone;
  }

  /**
   * Get stdout flag.
   *
   * @return bool
   *   Boolean flag.
   */
  public function isStandalone() {
    return $this->isStandalone;
  }

  /**
   * Set whether exporter must log to a file.
   *
   * @param bool $logToFile
   *   Log to file flag.
   */
  protected function setLogToFile(bool $logToFile) {
    $this->logToFile = $logToFile;
    $this->logMessage("Setting \$logToFile = " . ($logToFile ? "TRUE" : "FALSE"));
  }

  /**
   * Get whether exporter must log to a file.
   *
   * @return bool
   *   Boolean flag.
   */
  public function mustLogToFile() {
    return $this->logToFile;
  }

  /**
   * Set whether exporter must enable lock mode.
   *
   * @param bool $lock
   *   Lock mode flag.
   */
  protected function setLock(bool $lock) {
    $this->lock = $lock;
    $this->logMessage("Setting \$lock = " . ($lock ? "TRUE" : "FALSE"));
  }

  /**
   * Get lock mode flag.
   *
   * @return bool
   *   Boolean flag.
   */
  public function isLock() {
    return $this->lock;
  }

  /**
   * {@inheritdoc}
   */
  public function mustBuild(): bool {
    return $this->mustBuild;
  }

  /**
   * {@inheritdoc}
   */
  public function setMustBuild(bool $mustBuild) {
    $this->mustBuild = $mustBuild;
  }

  /**
   * Get the item being exported.
   *
   * @return mixed
   *   Exported item.
   */
  abstract protected function getExporterItem();

  /**
   * Get an id for the item being exported.
   *
   * @return string
   *   Exported item id.
   */
  abstract protected function getExporterItemId();

  /**
   * Get a label for the item being exported.
   *
   * @return string
   *   Exported item label.
   */
  abstract protected function getExporterItemLabel();

  /**
   * {@inheritdoc}
   */
  public function reset() {
    $this->operation = NULL;
    $this->timeStart = NULL;
    $this->timeEnd = NULL;
    $this->isMasterExport = TRUE;
    $this->isStandalone = FALSE;
    $this->options = NULL;
    $this->globalConfig = NULL;
    $this->outputConfig = NULL;
    $this->logToFile = TRUE;
    $this->lock = TRUE;
    $this->mustBuild = FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function makeSlaveOf(ExporterInterface $masterExporter) {
    $this->reset();
    $this->uniqueId = $masterExporter->uniqueId();
    $this->languageCode = $masterExporter->getLanguageCode();
    $this->isMasterExport = FALSE;
    $this->isStandalone = TRUE;
    $this->logToFile = $masterExporter->mustLogToFile();
    $this->lock = $masterExporter->isLock();
    $this->mustBuild = FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function setLanguageCode(string $languageCode) {
    $this->languageCode = strtolower($languageCode);
  }

  /**
   * {@inheritdoc}
   */
  public function getLanguageCode() {
    return $this->languageCode;
  }

  /**
   * Tells whether user running this process is valid.
   *
   * @param string $cliUser
   *   CLI username.
   *
   * @return bool
   *   TRUE if it's valid, FALSE otherwise.
   */
  public function isValidCliUser(string $cliUser): bool {
    // Ensure it is executed by a valid user on CLI.
    if ($this->staticSuiteUtils->isRunningOnCli()) {
      $allowedUsers = $this->configFactory->get('static_suite.settings')
        ->get('cli_allowed_users');
      if (count($allowedUsers) > 0 && !in_array($cliUser, $allowedUsers, TRUE)) {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   *
   */
  private function saveFile($url, $dir, $fileName) {
    if (!$file = file_get_contents($url)) {
      return FALSE;
    }

    $path = $dir . '/' . $fileName;
    return !(file_put_contents($path, $file) === FALSE);
  }

  /**
   * {@inheritdoc}
   */
  public function export(array $options = [], bool $isStandalone = FALSE, bool $logToFile = TRUE, bool $lock = TRUE, bool $build = TRUE) {

    // TODO - Remove languageCode as a mandatory option.
    /*if (!$this->languageCode) {
    throw new StaticSuiteUserException("Language code must be set before calling any exporter method");
    }*/

    $fileCollectionGroup = new FileCollectionGroup();

    // Ensure it is executed by a valid user on CLI.
    $currentUser = posix_getpwuid(posix_geteuid())['name'];
    if (!$this->isValidCliUser($currentUser)) {
      if (($this->staticSuiteUtils->isInteractiveTty() && $this->configFactory->get('static_suite.settings')->get('cli_throw_exception_interactive_tty')) ||
        (!$this->staticSuiteUtils->isInteractiveTty() && $this->configFactory->get('static_suite.settings')->get('cli_throw_exception_non_interactive_tty'))) {
        throw new StaticSuiteUserException('User "' . $currentUser . '" is not allowed to execute this process on CLI.');
      }
      $this->resultingFileCollectionGroup = $fileCollectionGroup;
      return $this->resultingFileCollectionGroup;
    }

    // If a variant is present, ensure it's not executed inside a master export.
    if ($this->isMasterExport() && !empty($options['variant'])) {
      throw new StaticSuiteUserException('Exporting a variant outside of a child export process is not allowed. Found "' . $options['variant'] . '" variant.');
    }

    try {
      // Early check for opting out of this process.
      $event = $this->dispatchEvent(StaticExportEvents::PREFLIGHT);
      if ($event->mustAbort()) {
        $this->resultingFileCollectionGroup = $fileCollectionGroup;
        return $this->resultingFileCollectionGroup;
      }

      // Initialize.
      if ($this->isMasterExport) {
        $this->uniqueId = $this->uniqueIdHelper->generateUniqueId();
      }
      $this->startBenchmark();
      $this->operation = ExporterInterface::OPERATION_WRITE;

      // Enable/disable logging as requested.
      $this->setLogToFile($logToFile);

      // Start logging.
      $this->logMessage("Exporter " . get_class($this) . " started with UNIQUE ID " . $this->uniqueId());
      $this->logMessage("OPERATION: " . $this->operation);
      $this->logMessage("Received \$options: \n" . $this->optionsForLogging($options));
      $this->logMessage("Received \$isStandalone = " . ($isStandalone ? "TRUE" : "FALSE"));
      $this->logMessage("Received \$logToFile = " . ($logToFile ? "TRUE" : "FALSE"));
      $this->logMessage("Received \$lock = " . ($lock ? "TRUE" : "FALSE"));
      $this->logMessage("Received \$this->mustBuild = " . ($this->mustBuild ? "TRUE" : "FALSE"));

      $this->dispatchEvent(StaticExportEvents::STARTS);

      $this->logMessage("preProcessOptions() starts");
      $options = $this->preProcessOptions($options);
      $this->logMessage("preProcessOptions() done.");

      $this->dispatchEvent(StaticExportEvents::CHECK_PARAMS_STARTS);
      $this->logMessage("checkParams() starts");
      if ($this->checkParams($options, $isStandalone)) {
        $this->logMessage("Setting params.");
        $this->setOptions($options);
        $this->setIsStandalone($isStandalone);
        $this->setLogToFile($logToFile);
        $this->setLock($lock);
        $this->logMessage("Setting params done.");
      }
      $this->logMessage("checkParams() done.");
      $this->dispatchEvent(StaticExportEvents::CHECK_PARAMS_ENDS);

      // Check if received item is exportable.
      if (!$this->isExportable($this->getOptions())) {
        throw new StaticSuiteUserException("Item '" . $this->getExporterItemId() . "' ('" . $this->getExporterItemLabel() . "') is not exportable.");
      }

      $calculatedOutputConfig = $this->calculateOutputConfig();
      // Early opt-out if no config is defined.
      if ($calculatedOutputConfig !== NULL) {
        $this->outputConfig = $calculatedOutputConfig;

        // Save a pointer in FileCollectionWriter queue system to ensure
        // concurrent exports are committed to data directory in order.
        if ($this->isMasterExport) {
          $this->fileCollectionWriter->setExporter($this);
          // Only insert into the queue if $lock is true.
          if ($lock) {
            $this->logMessage("Queue insertion starting...");
            $this->fileCollectionWriter->startQueueInsertion();
            $this->logMessage("Queue insertion started.");
          }
        }

        $dataToExport = $this->handleResolver();
        if (is_string($dataToExport)) {
          $formattedData = $dataToExport;
        }
        else {
          $formattedData = $this->handleFormatter($dataToExport);
        }
        $fileCollectionGroup = $this->handleOutput($formattedData);
      }

      $this->dispatchEvent(StaticExportEvents::ENDS);
    }
    catch (Throwable $e) {
      // This is Throwable to capture any kind of exception and ensure
      // that any pending queue insertion is deleted.
      if ($this->isMasterExport) {
        $this->fileCollectionWriter->deletePendingQueueInsertion();
      }

      // Log the error to the file log.
      $this->logMessage($e->getMessage() . "\n" . $e->getTraceAsString());

      // Send an error to logger.
      // Do not log anything when running on an interactive TTY.
      if (!$this->staticSuiteUtils->isRunningOnCli() || !$this->staticSuiteUtils->isInteractiveTty()) {
        $message = $e->getMessage() . ' Please, review ' . $this->configFactory->get('static_export.settings')->get('log_dir') . '/' . $this->uniqueId . '.log';
        $this->getLogger('static_export')->error($message);
      }

      // Rethrow StaticSuiteUserException.
      if ($e instanceof StaticSuiteUserException) {
        throw new StaticSuiteUserException($e->getMessage(), $e->getCode(), $e);
      }

      // Any other exception is rethrown as StaticSuiteException.
      throw new StaticSuiteException($e->getMessage(), $e->getCode(), $e);
    }

    $this->resultingFileCollectionGroup = $fileCollectionGroup;

    $this->endBenchmark();
    return $this->resultingFileCollectionGroup;
  }

  /**
   * {@inheritdoc}
   *
   * TODO - Remove this method to avoid duplicated code.
   */
  public function delete(array $options = [], bool $isStandalone = FALSE, bool $logToFile = TRUE, bool $lock = TRUE) {
    // TODO - Remove languageCode as a mandatory option.
    /* if (!$this->languageCode) {
    throw new StaticSuiteUserException("Language code must be set before calling any exporter method");
    }*/

    $fileCollectionGroup = new FileCollectionGroup();

    // Ensure it is executed by a valid user on CLI.
    $currentUser = posix_getpwuid(posix_geteuid())['name'];
    if (!$this->isValidCliUser($currentUser)) {
      if (($this->staticSuiteUtils->isInteractiveTty() && $this->configFactory->get('static_suite.settings')->get('cli_throw_exception_interactive_tty')) ||
        (!$this->staticSuiteUtils->isInteractiveTty() && $this->configFactory->get('static_suite.settings')->get('cli_throw_exception_non_interactive_tty'))) {
        throw new StaticSuiteUserException('User "' . $currentUser . '" is not allowed to execute this process on CLI.');
      }
      $this->resultingFileCollectionGroup = $fileCollectionGroup;
      return $this->resultingFileCollectionGroup;
    }

    try {
      // Early check for opting out of this process.
      $event = $this->dispatchEvent(StaticExportEvents::PREFLIGHT);
      if ($event->mustAbort()) {
        $this->resultingFileCollectionGroup = $fileCollectionGroup;
        return $this->resultingFileCollectionGroup;
      }

      // Initialize.
      if ($this->isMasterExport) {
        $this->uniqueId = $this->uniqueIdHelper->generateUniqueId();
      }
      $this->startBenchmark();
      $this->operation = ExporterInterface::OPERATION_DELETE;

      // Enable/disable logging as requested.
      $this->setLogToFile($logToFile);

      // Start logging.
      $this->logMessage("Exporter " . get_class($this) . " started with UNIQUE ID " . $this->uniqueId());
      $this->logMessage("OPERATION: " . $this->operation);
      $this->logMessage("Received \$options: \n" . $this->optionsForLogging($options));
      $this->logMessage("Received \$isStandalone = " . ($isStandalone ? "TRUE" : "FALSE"));
      $this->logMessage("Received \$logToFile = " . ($logToFile ? "TRUE" : "FALSE"));
      $this->logMessage("Received \$lock = " . ($lock ? "TRUE" : "FALSE"));
      $this->logMessage("Received \$this->mustBuild = " . ($this->mustBuild ? "TRUE" : "FALSE"));

      $this->dispatchEvent(StaticExportEvents::STARTS);

      $this->logMessage("preProcessOptions() starts");
      $options = $this->preProcessOptions($options);
      $this->logMessage("preProcessOptions() done.");

      $this->dispatchEvent(StaticExportEvents::CHECK_PARAMS_STARTS);
      $this->logMessage("checkParams() starts");
      if ($this->checkParams($options, $isStandalone)) {
        $this->logMessage("Setting params.");
        $this->setOptions($options);
        $this->setIsStandalone($isStandalone);
        $this->setLogToFile($logToFile);
        $this->setLock($lock);
        $this->logMessage("Setting params done.");
      }
      $this->logMessage("checkParams() done.");
      $this->dispatchEvent(StaticExportEvents::CHECK_PARAMS_ENDS);

      $calculatedConfig = $this->calculateOutputConfig();
      // Early opt-out if no config is defined.
      if ($calculatedConfig !== NULL) {
        $this->outputConfig = $calculatedConfig;

        // Save a pointer in FileCollectionWriter queue system to ensure
        // concurrent exports are committed to data directory in order.
        if ($this->isMasterExport) {
          $this->fileCollectionWriter->setExporter($this);
          // Only insert into the queue if $lock is true.
          if ($lock) {
            $this->logMessage("Queue insertion starting...");
            $this->fileCollectionWriter->startQueueInsertion();
            $this->logMessage("Queue insertion started.");
          }
        }

        // $dataToExport = $this->handleResolver();
        // $formattedData = $this->handleFormatter($dataToExport);
        $fileCollectionGroup = $this->handleOutput();
      }

      $this->dispatchEvent(StaticExportEvents::ENDS);

    }
    catch (Throwable $e) {
      // This is Throwable to capture any kind of exception and ensure
      // that any pending queue insertion is deleted.
      if ($this->isMasterExport) {
        $this->fileCollectionWriter->deletePendingQueueInsertion();
      }

      // Log the error to the file log.
      $this->logMessage($e->getMessage() . "\n" . $e->getTraceAsString());

      // Send an error to logger.
      // Do not log anything when running on an interactive TTY.
      if (!$this->staticSuiteUtils->isRunningOnCli() || !$this->staticSuiteUtils->isInteractiveTty()) {
        $message = $e->getMessage() . " Please, review " . $this->configFactory->get('static_export.settings')->get('log_dir') . '/' . $this->uniqueId . '.log';
        $this->getLogger('static_export')->error($message);
      }

      // Rethrow StaticSuiteUserException.
      if ($e instanceof StaticSuiteUserException) {
        throw new StaticSuiteUserException($e->getMessage(), $e->getCode(), $e);
      }

      // Any other exception is rethrown as StaticSuiteException.
      throw new StaticSuiteException($e->getMessage(), $e->getCode(), $e);
    }

    $this->resultingFileCollectionGroup = $fileCollectionGroup;

    $this->endBenchmark();
    return $this->resultingFileCollectionGroup;
  }

  /**
   * Get the resulting FileCollectionGroup.
   *
   * @return \Drupal\static_export\File\FileCollectionGroup
   *   The resulting FileCollectionGroup.
   */
  public function getResultingFileCollectionGroup() {
    return $this->resultingFileCollectionGroup;
  }

  /**
   * Get output config data from a specific exporter.
   *
   * Variants are not defined here but in calculateOutputConfig().
   *
   * @return \Drupal\static_export\Plugin\ExporterOutputConfig|null
   *   Output config data.
   */
  abstract protected function getOutputDefinition(): ?ExporterOutputConfig;

  /**
   * Get exported data from the specific resolver of a exporter.
   *
   * @return array|string
   *   Exported data.
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  abstract protected function calculateDataFromResolver();

  /**
   * Calculate output config.
   *
   * @return \Drupal\static_export\Plugin\ExporterOutputConfig|null
   *   Output config data.
   */
  protected function calculateOutputConfig(): ?ExporterOutputConfig {
    $this->logMessage("Calculating output config...");
    $this->dispatchEvent(StaticExportEvents::CONFIG_STARTS);
    $outputConfig = $this->getOutputDefinition();
    // TODO CHECK IF DISPATCHING NULL BREAKS SOMETHING
    // Avoid dispatching an event with null data, that could lead to type errors.
    if ($outputConfig === NULL) {
      return NULL;
    }
    $event = $this->dispatchEvent(StaticExportEvents::CONFIG_ENDS, [StaticExportEvent::EVENT_CONFIG => $outputConfig]);
    $outputConfig = $event->getOutputConfig();

    // If a variant is present, we ensure it's added at the end of the filename.
    if (!empty($this->options['variant'])) {
      $outputConfig = new ExporterOutputConfig(
        $outputConfig->getDir(),
        $outputConfig->getFilename() . '.' . $this->options['variant'],
        $outputConfig->getExtension(),
        $outputConfig->getFormat()
      );
    }
    $this->logMessage("Calculated output config: " . print_r($outputConfig, TRUE));
    return $outputConfig;
  }

  /**
   * Handles the process of getting export data using a specific resolver.
   *
   * @return array|string
   *   Data to be exported.
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  protected function handleResolver() {
    $this->logMessage("Handling resolver.");
    $this->dispatchEvent(StaticExportEvents::RESOLVER_STARTS);
    $this->dataFromResolver = $this->calculateDataFromResolver();
    $event = $this->dispatchEvent(StaticExportEvents::RESOLVER_ENDS, [StaticExportEvent::EVENT_DATA_FROM_RESOLVER => $this->dataFromResolver]);
    $this->dataFromResolver = $event->getDataFromResolver();
    $this->logMessage("Handling resolver done.");
    return $this->dataFromResolver;
  }

  /**
   * Gets DataFromResolver.
   * TODO Ensure this method is really needed.
   */
  public function getDataFromResolver() {
    return $this->dataFromResolver;
  }

  /**
   * Handles data formatting.
   *
   * @param array $data
   *   Entity data.
   *
   * @return string
   *   Formatted entity data.
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  protected function handleFormatter(array $data) {
    $this->logMessage("Handling formatter.");
    $this->dispatchEvent(StaticExportEvents::FORMATTER_STARTS);

    $formatId = $this->outputConfig->getFormat();
    if (empty($formatId)) {
      throw new StaticSuiteUserException("No format defined for this data.");
    }

    try {
      $outputFormatter = $this->outputFormatterManager->getInstance(['plugin_id' => $formatId]);
    }
    catch (PluginException $e) {
      throw new StaticSuiteUserException("Unknown format: " . $formatId);
    }

    // Allow exporters to bypass formatting by adding a special key to the data
    // array.
    if (isset($data[ExporterInterface::OVERRIDE_FORMAT])) {
      $dataFromFormatter = $data[ExporterInterface::OVERRIDE_FORMAT];
    }
    else {
      $dataFromFormatter = $outputFormatter->format($data);
    }

    $event = $this->dispatchEvent(StaticExportEvents::FORMATTER_ENDS, [StaticExportEvent::EVENT_DATA_FROM_FORMATTER => $dataFromFormatter]);
    $dataFromFormatter = $event->getDataFromFormatter();

    $this->logMessage("Handling formatter done.");
    return $dataFromFormatter;
  }

  /**
   * Handles data output.
   *
   * @param string $formattedData
   *   The data to be saved.
   *
   * @return \Drupal\static_export\File\FileCollectionGroup
   *   A FileCollectionGroup. It could be more than one FileCollection because
   *   the fileCollectionWriter manages a queue.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  protected function handleOutput(string $formattedData = "") {
    $this->logMessage("Handling output.");
    $this->dispatchEvent(StaticExportEvents::OUTPUT_STARTS);

    // Get all data for the FileItem.
    $fileCollection = new FileCollection($this->uniqueId);
    $fileOperation = ($this->operation == ExporterInterface::OPERATION_WRITE) ? FileItem::OPERATION_WRITE : FileItem::OPERATION_DELETE;
    $filePath = $this->getAbsoluteFilePath();
    $exporterItem = $this->getExporterItem();
    $currentStatus = 1;
    $oldStatus = 1;
    if ($exporterItem instanceof EditorialContentEntityBase) {
      $currentStatus = $exporterItem->isPublished() ? 1 : 0;
      if (is_object($exporterItem->original) && method_exists($exporterItem->original, 'isPublished')) {
        $oldStatus = $exporterItem->original->isPublished() ? 1 : 0;
      }
      else {
        // When previewing a node using "static_build.preview" route, no
        // original entity is available, since no save has been done. In this
        // case, no status has changed, so keep $oldStatus and $currentStatus
        // the same.
        $oldStatus = $currentStatus;
      }
    }
    $fileCollection->addFileItem(new FileItem(
      $fileOperation,
      $currentStatus,
      $oldStatus,
      $filePath,
      $formattedData,
      $this->getExporterItemId(),
      $this->getExporterItemLabel(),
      // Call getBenchmark() here, before dispatching any event,
      // to get real data for this file.
      $this->getBenchmark()
    ));

    $event = $this->dispatchEvent(StaticExportEvents::WRITE_STARTS, [StaticExportEvent::EVENT_FILE_COLLECTION => $fileCollection]);
    $fileCollection = $event->getFileCollection();

    // Initialize $fileCollectionGroup.
    $fileCollectionGroup = new FileCollectionGroup($fileCollection);

    // Only write data if this is the master export (the first one)
    if ($this->isMasterExport && $fileCollection->size() > 0) {
      // We get an array of FileCollection because fileCollectionWriter
      // processes a queue.
      $fileCollectionGroup = $this->fileCollectionWriter->save($fileCollection);
    }

    $event = $this->dispatchEvent(StaticExportEvents::WRITE_ENDS, [StaticExportEvent::EVENT_FILE_COLLECTION_GROUP => $fileCollectionGroup]);
    $fileCollectionGroup = $event->getFileCollectionGroup();

    $event = $this->dispatchEvent(StaticExportEvents::OUTPUT_ENDS, [StaticExportEvent::EVENT_FILE_COLLECTION_GROUP => $fileCollectionGroup]);
    $fileCollectionGroup = $event->getFileCollectionGroup();

    $this->logMessage("Handling output done.");
    return $fileCollectionGroup;
  }

  /**
   * {@inheritdoc}
   */
  public function getAbsoluteFilePath(): ?string {
    if (!$this->outputConfig) {
      $this->outputConfig = $this->calculateOutputConfig();
    }
    return $this->outputConfig ? $this->outputConfig->__toString() : NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getRelativeFilePath(): ?string {
    $absoluteFilePath = $this->getAbsoluteFilePath();
    if (!$absoluteFilePath) {
      return NULL;
    }
    $dataDir = $this->configFactory->get('static_export.settings')->get('data_dir');
    return str_replace($dataDir, "", $absoluteFilePath);
  }

  /**
   * {@inheritdoc}
   */
  public function logMessage(string $message):void {
    if ($this->logToFile && $this->uniqueId()) {
      $logFile = $this->configFactory->get('static_export.settings')->get('log_dir') . "/" . $this->uniqueId() . '.log';
      $timeStamp = $this->staticSuiteUtils->getFormattedMicroDate("Y-m-d H:i:s.u");
      $indent = $this->isMasterExport ? "" : "[sub-export] ";
      $line = "[$timeStamp] $indent$message\n";
      $logDir = dirname($logFile);
      if (!file_exists($logDir)) {
        mkdir(dirname($logFile), 0777, TRUE);
      }
      file_put_contents($logFile, $line, FILE_APPEND);
    }
  }

  /**
   * Get logger object.
   *
   * @return \Psr\Log\LoggerInterface
   *   A logger object
   */
  public function logger(): LoggerInterface {
    return $this->getLogger('static_export');
  }

  /**
   * Returns output config.
   *
   * @return \Drupal\static_export\Plugin\ExporterOutputConfig
   *   Config data.
   */
  public function getOutputConfig(): ExporterOutputConfig {
    return $this->outputConfig;
  }

  /**
   * Dispatch StaticExportEvent for the given tag.
   *
   * @param string $eventName
   *   The name of the event.
   * @param array $data
   *   Data for the event.
   *
   * @return \Drupal\static_export\Event\StaticExportEvent
   *   The event.
   */
  public function dispatchEvent(string $eventName, array $data = []): StaticExportEvent {
    $event = new StaticExportEvent($this);
    $event->setData($data);

    // Dispatch the event.
    $this->logMessage("EVENT DISPATCH '$eventName' TRIGGERED");
    $this->eventDispatcher->dispatch($eventName, $event);
    $this->logMessage("EVENT DISPATCH '$eventName' DONE");

    // Return the event.
    return $event;
  }

  /**
   * Tell whether this exporter should always write.
   *
   * @return bool
   *   True if write is forced.
   */
  public function isForceWrite(): bool {
    return $this->isForceWrite;
  }

  /**
   * Flag to indicate that this exporter should always write.
   *
   * @param bool $isForceWrite
   *   Flag for always write.
   */
  public function setIsForceWrite(bool $isForceWrite) {
    $this->isForceWrite = $isForceWrite;
  }

  /**
   * Starts benchmark.
   */
  public function startBenchmark() {
    $this->timeStart = microtime(TRUE);
    $this->timeEnd = NULL;
  }

  /**
   * Ends benchmark.
   */
  public function endBenchmark() {
    $this->timeEnd = microtime(TRUE);
  }

  /**
   * Get elapsed time.
   *
   * @return float
   *   Elapsed time.
   */
  public function getBenchmark() {
    $timeEnd = $this->timeEnd ? $this->timeEnd : microtime(TRUE);
    return number_format(round($timeEnd - $this->timeStart, 3), 3);
  }

  /**
   * Print options for logging.
   *
   * @param array $options
   *   Options array to log.
   *
   * @return string
   *   A loggable string.
   */
  private function optionsForLogging(array $options) {
    $loggableString = "";
    foreach ($options as $key => $option) {
      $loggableString .= "$key => ";
      if (is_object($option)) {
        $loggableString .= get_class($option);
        if (method_exists($option, "id")) {
          $loggableString .= " ID: " . $option->id();
        }
      }
      else {
        $loggableString .= print_r($option, TRUE);
      }
      $loggableString .= "\n";
    }
    return "\n" . trim($loggableString);
  }

  /**
   * {@inheritdoc}
   *
   * By default, this method always returns true. Some exporters do not
   * accept any parameter (they export a fixed set of data) so they don't
   * need to check any option.
   */
  public function isExportable(array $options): bool {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getVariantKeys(): array {
    return [];
  }

}
