<?php

namespace Drupal\static_export\Plugin;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;

/**
 * Defines an interface for exporting data to static files.
 */
interface ExporterInterface extends PluginInspectionInterface, ContainerFactoryPluginInterface {

  public const WORK_DIR = '.work';
  public const OPERATION_WRITE = 'static_export:write';
  public const OPERATION_DELETE = 'static_export:delete';
  public const OVERRIDE_FORMAT = '___EXPORTER_OVERRIDE_FORMAT___';

  /**
   * Set main language code.
   *
   * This is required for nearly all exporter operations, so we hold it here.
   *
   * @param string $languageCode
   *   A language code.
   */
  public function setLanguageCode(string $languageCode);

  /**
   * Get main language code.
   *
   * @return string
   *   The language code.
   */
  public function getLanguageCode();

  /**
   * Set options array.
   *
   * @param array $options
   *   Options for this operation.
   */
  public function setOptions(array $options): void;

  /**
   * Get options array.
   *
   * @return array
   *   Options for this operation.
   */
  public function getOptions(): array;

  /**
   * Get operation (write / delete)
   *
   * @todo - make this a simple option
   *
   * @return string
   *   Operation mode.
   */
  public function getOperation(): string;

  /**
   * Checks incoming params.
   *
   * Every exporter should check its own params and throw a exception
   * on any problem.
   *
   * @param array $options
   *   Options for this operation.
   * @param bool $isStandalone
   *   Flag for standalone mode.
   *
   * @return bool
   *   True if params are ok.
   *
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public function checkParams(array $options, bool $isStandalone = FALSE);

  /**
   * Tell whether exporter must trigger a build.
   *
   * @return bool
   *   Build flag.
   */
  public function mustBuild(): bool;

  /**
   * Set whether exporter must trigger a build.
   *
   * @param bool $mustBuild
   *   Build flag.
   */
  public function setMustBuild(bool $mustBuild);

  /**
   * Exports a single item.
   *
   * @param array $options
   *   Options for export operation.
   * @param bool $isStandalone
   *   Flag for standalone mode.
   * @param bool $logToFile
   *   Whether it should log data to a logfile.
   * @param bool $lock
   *   Whether it should enable locking for disk operations.
   *
   * @return \Drupal\static_export\File\FileCollectionGroup
   *   A FileCollectionGroup of FileCollections with FileItems.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   * @throws \Drupal\static_suite\StaticSuiteUserException *
   */
  public function export(array $options = [], bool $isStandalone = FALSE, bool $logToFile = TRUE, bool $lock = TRUE);

  /**
   * Deletes a single item.
   *
   * @param array $options
   *   Options for delete operation.
   * @param bool $isStandalone
   *   Flag for standalone mode.
   * @param bool $logToFile
   *   Whether it should log data to a logfile.
   * @param bool $lock
   *   Whether it should enable locking for disk operations.
   *
   * @return \Drupal\static_export\File\FileCollectionGroup
   *   A FileCollectionGroup of FileCollection with FileItems.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   * @throws \Drupal\static_suite\StaticSuiteUserException
   */
  public function delete(array $options = [], bool $isStandalone = FALSE, bool $logToFile = TRUE, bool $lock = TRUE);

  /**
   * Logs a message.
   *
   * @param string $message
   *   A message to be logged.
   */
  public function logMessage(string $message): void;

  /**
   * Resets the exporter.
   */
  public function reset();

  /**
   * Make this exporter slave of another one.
   *
   * Set and copy some values to ensure proper execution on sub exports.
   *
   * @param ExporterInterface $masterExporter
   *   The master exporter instance.
   */
  public function makeSlaveOf(ExporterInterface $masterExporter);

  /**
   * Tells whether an exporter considers an item exportable.
   *
   * This method is used in two places:
   *  1) Internally, inside a exporter, to avoid exporting something not
   *     allowed. An exception is thrown if that happens.
   *  2) Outside exporters, as an early check to avoid passing wrong parameters
   *     to them. This way, no exception is thrown.
   *
   * @param array $options
   *   An array of options, just like the one received by export() method.
   *
   * @return bool
   *   True if it's exportable
   */
  public function isExportable(array $options): bool;

  /**
   * Get exported absolute file path based on config.
   *
   * It can be obtained without actually running the exporter.
   *
   * @return string|null
   *   Exported absolute file path.
   */
  public function getAbsoluteFilePath(): ?string;

  /**
   * Get exported relative file path based on config.
   *
   * It can be obtained without actually running the exporter.
   *
   * @return string|null
   *   Exported relative file path.
   */
  public function getRelativeFilePath(): ?string;

  /**
   * Get variant keys of a exporter.
   *
   * Each exporter could implement a different strategy to determine its
   * variant keys, if any. For example, EntityExporter uses its own
   * StaticResolver plugins, while other exporters, like TranslationExporter,
   * could always export a fixed variant with a subset of locales.
   *
   * This method returns an empty array because most exporters don't make use
   * of variants.
   *
   * @return array
   */
  public function getVariantKeys(): array;

}
