<?php

namespace Drupal\static_export\Plugin\OutputFormatter;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\static_export\Plugin\OutputFormatterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Serializer\SerializerInterface;

/**
 * Provides a XML output formatter.
 *
 * @StaticOutputFormatter(
 *  id = "xml",
 *  label = "XML",
 *  description = "XML output formatter",
 *  extension = "xml",
 * )
 */
class XmlOutputFormatter extends PluginBase implements OutputFormatterInterface, ContainerFactoryPluginInterface {

  /**
   * Serializer.
   *
   * @var \Symfony\Component\Serializer\Serializer
   */
  protected $serializer;

  /**
   * Constructor for exporter.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Symfony\Component\Serializer\SerializerInterface $serializer
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    SerializerInterface $serializer
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->serializer = $serializer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get("serializer")
    );
  }

  /**
   * {@inheritdoc}
   */
  public function format(array $data): string {
    // Avoid errors with missing namespaces.
    $data = $this->changeKey($data, function ($key, $value) {
      return str_replace(':', '_', $key);
    });

    return $this->serializer->serialize($data, 'xml');
  }

  /**
   * Recursively changes array keys.
   *
   * @param array $arr
   *   Array to ve converted.
   * @param array $keySetOrCallBack
   *   Key set (['old-key' => 'new-key']) or callback function to be executed
   *   for each array item.
   *
   * @return array
   */
  protected function changeKey(array $arr, $keySetOrCallBack = []) {
    $newArr = [];
    foreach ($arr as $k => $v) {
      if (is_callable($keySetOrCallBack)) {
        $key = call_user_func_array($keySetOrCallBack, [$k, $v]);
      }
      else {
        $key = $keySetOrCallBack[$k] ?? $k;
      }
      $newArr[$key] = is_array($v) ? $this->changeKey($v, $keySetOrCallBack) : $v;
    }
    return $newArr;
  }

}
