<?php

namespace Drupal\static_export\Semaphore;

use Drupal\static_suite\StaticSuiteException;
use Exception;

/**
 * A service for handling semaphores.
 */
class Semaphore {
  /**
   * A System V semaphore id.
   *
   * @var resource
   */
  private $semId;

  /**
   * A flag to set if a semaphore has been acquired.
   *
   * @var bool
   */
  private $isAcquired = FALSE;

  /**
   * Semaphore constructor.
   *
   * @param int $key
   *   Semaphore Id.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function __construct(int $key) {
    if (!($this->semId = sem_get($key, 1))) {
      throw new StaticSuiteException("Error getting semaphore ID");
    }
  }

  /**
   * Acquires a semaphore.
   *
   * @param bool $noWait
   *   Specifies if the process shouldn't wait for the semaphore to be acquired.
   *   If set to true, the call will return false immediately if a semaphore
   *   cannot be immediately acquired.
   *
   * @return bool
   *   True if it's acquired or if where are acquiring it right now.
   *
   * @throws StaticSuiteException
   */
  public function acquire($noWait = FALSE) {
    if (!$this->isAcquired) {
      $this->isAcquired = sem_acquire($this->semId, $noWait);
      if (!$this->isAcquired && $noWait === FALSE) {
        throw new StaticSuiteException("Error acquiring semaphore");
      }
    }
    return $this->isAcquired;
  }

  /**
   * Releases a semaphore.
   */
  public function release() {
    if (!$this->isAcquired) {
      return;
    }

    try {
      sem_release($this->semId);
    }
    catch (Exception $e) {
      // Do nothing.
    }

    $this->isAcquired = FALSE;
  }

  /**
   * Get a semaphore status.
   *
   * @return bool
   *   True if it's acquired.
   */
  public function isAcquired() {
    return $this->isAcquired;
  }

}
