<?php

namespace Drupal\static_export\Service;

use Drupal\Core\Render\Markup;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\static_export\File\FileCollection;
use Drupal\static_export\File\FileCollectionFormatter;
use Drupal\static_export\File\FileCollectionGroup;

/**
 * Messenger service for showing messages to Drupal users.
 */
class MessengerService {

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * The File Collection Formatter.
   *
   * @var \Drupal\static_export\File\FileCollectionFormatter
   */
  protected $fileCollectionFormatter;

  /**
   * Constructs a new EntityReferenceUsage object.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Messenger service.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\static_export\File\FileCollectionFormatter $fileCollectionFormatter
   *   The File Collection Formatter.
   */
  public function __construct(MessengerInterface $messenger, AccountProxyInterface $currentUser, FileCollectionFormatter $fileCollectionFormatter) {
    $this->messenger = $messenger;
    $this->currentUser = $currentUser;
    $this->fileCollectionFormatter = $fileCollectionFormatter;
  }

  /**
   * Shows a message if user is allowed to view it.
   *
   * @param string $message
   *   A message.
   *
   * @return bool
   *   True if user is allowed to be view it.
   */
  public function addMessage(string $message) {
    if ($this->currentUser->hasPermission('view static export logs')) {
      $this->messenger->addMessage($message);
    }
    return FALSE;
  }

  /**
   * Shows an error .
   *
   * @param string $message
   *   A message.
   */
  public function addError(string $message) {
    $this->messenger->addError($message);
  }

  /**
   * Shows a message coming from a FileCollection.
   *
   * @param \Drupal\static_export\File\FileCollection $fileCollection
   *   A FileCollection.
   *
   * @return bool
   *   True if user is allowed to be view it.
   */
  protected function addFileCollection(FileCollection $fileCollection) {
    if ($this->currentUser->hasPermission('view static export logs')) {
      $this->fileCollectionFormatter->setFileCollection($fileCollection);
      $htmlLines = $this->fileCollectionFormatter->getHtmlLines($this->currentUser);
      foreach ($htmlLines as $line) {
        $renderedMessage = Markup::create($line);
        $this->messenger->addStatus($renderedMessage);
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Shows a message coming from an array of FileCollection.
   *
   * @param \Drupal\static_export\File\FileCollectionGroup $fileCollectionGroup
   *   A FileCollectionGroup.
   */
  public function addFileCollectionGroup(FileCollectionGroup $fileCollectionGroup) {
    if (!$fileCollectionGroup->isEmpty() && $this->currentUser->hasPermission('view static export logs')) {
      $this->messenger->addStatus("Static Export operations:");
      foreach ($fileCollectionGroup->getFileCollections() as $delta => $fileCollection) {
        $this->messenger->addStatus("#" . ($delta + 1) . " File Collection " . $fileCollection->uniqueId());
        $this->messenger->addStatus("============================================================================", TRUE);
        $this->addFileCollection($fileCollection);
      }
    }
  }

}
