<?php

namespace Drupal\static_export\Service;

use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Path\AliasManager;
use Drupal\Core\Url;
use Drupal\static_export\Plugin\ExporterManagerInterface;
use Throwable;

/**
 * Path resolver service for exported nodes.
 */
class NodePathResolverService {

  /**
   * The Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The alias manager.
   *
   * @var \Drupal\Core\Path\AliasManager
   */
  protected $aliasManager;

  /**
   * Exporter Manager.
   *
   * @var \Drupal\static_export\Plugin\ExporterManagerInterface
   */
  protected $exporterManager;

  /**
   * PathResolver service constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   Drupal entity manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\Core\Path\AliasManager $aliasManager
   *   The alias manager.
   * @param \Drupal\static_export\Plugin\ExporterManagerInterface $exporterManager
   *   Exporter Manager.
   */
  public function __construct(
    EntityTypeManager $entityTypeManager,
    LanguageManagerInterface $languageManager,
    AliasManager $aliasManager,
    ExporterManagerInterface $exporterManager
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->languageManager = $languageManager;
    $this->aliasManager = $aliasManager;
    $this->exporterManager = $exporterManager;
  }

  /**
   * Given a node id, return its exported relative file path.
   *
   * @param string $nid
   *   Node id to search for.
   *
   * @return string
   *   Node's exported relative file path.
   */
  public function getExportedFilePathRelativeByNodeId(string $nid) {
    $relativePath = NULL;
    try {
      $node = $this->entityTypeManager->getStorage('node')->load($nid);
    }
    catch (Throwable $e) {
      $node = NULL;
    }
    if ($node) {
      $languageCode = $this->languageManager->getDefaultLanguage()->getId();
      $entityExporter = $this->exporterManager->getInstance(['plugin_id' => 'entity']);
      $entityExporter->setLanguageCode($languageCode);
      $entityExporter->setOptions([$entityExporter::PARAM_ENTITY => $node]);
      $relativePath = $entityExporter->getRelativeFilePath();
    }
    return $relativePath;
  }

  /**
   * Given a node id, return its exported absolute file path.
   *
   * @param string $nid
   *   Node id to search for.
   *
   * @return string
   *   Node's exported absolute file path.
   */
  public function getExportedFilePathAbsoluteByNodeId(string $nid) {
    $absolutePath = NULL;
    try {
      $node = $this->entityTypeManager->getStorage('node')->load($nid);
      $languageCode = $this->languageManager->getDefaultLanguage()->getId();
      $entityExporter = $this->exporterManager->getInstance(['plugin_id' => 'entity']);
      $entityExporter->setLanguageCode($languageCode);
      $entityExporter->setOptions([$entityExporter::PARAM_ENTITY => $node]);
      $absolutePath = $entityExporter->getAbsoluteFilePath();
    }
    catch (Throwable $e) {
      // Do nothing.
    }
    return $absolutePath;
  }

  /**
   * Given a node url, return node id.
   *
   * @param string $nodeUrl
   *   Node url.
   *
   * @return int
   *   Node id.
   */
  public function getNodeIdByUrl(string $nodeUrl) {
    $nid = NULL;
    try {
      // $nodeUrl could arrive without a leading slash, but
      // $this->aliasManager (Drupal\Core\Path\AliasManager) needs that leading
      // slash.
      if ($nodeUrl[0] != "/") {
        $nodeUrl = "/" . $nodeUrl;
      }
      $alias = $this->aliasManager->getPathByAlias($nodeUrl, $this->languageManager->getDefaultLanguage()->getId());
      $params = Url::fromUri("internal:" . $alias)->getRouteParameters();
      $entityType = key($params);
      if ($entityType == 'node') {
        $nid = $params['node'];
      }
    }
    catch (Throwable $e) {
      // Do nothing.
    }
    return $nid;
  }

  /**
   * Given a node url, return its exported relative file path.
   *
   * @param string $nodeUrl
   *   Node url.
   *
   * @return string
   *   Node's exported relative file path.
   */
  public function getExportedFilePathRelativeByNodeUrl(string $nodeUrl) {
    $nid = $this->getNodeIdByUrl($nodeUrl);
    $relativePath = NULL;
    if ($nid) {
      $relativePath = $this->getExportedFilePathRelativeByNodeId($nid);
    }
    return $relativePath;
  }

  /**
   * Given a node url, it returns its exported absolute file path.
   *
   * @param string $nodeUrl
   *   Node url.
   *
   * @return string
   *   Node's exported absolute file path.
   */
  public function getExportedFilePathAbsoluteByNodeUrl(string $nodeUrl) {
    $nid = $this->getNodeIdByUrl($nodeUrl);
    $absolutePath = NULL;
    if ($nid) {
      $absolutePath = $this->getExportedFilePathAbsoluteByNodeId($nid);
    }
    return $absolutePath;
  }

}
