<?php

namespace Drupal\static_preview_gatsby_instant\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\static_build\Plugin\StaticBuilderInterface;
use Drupal\static_build\Plugin\StaticBuilderManagerInterface;
use Drupal\static_preview_gatsby_instant\Mocker\GatsbyMocker;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Throwable;

/**
 * Gatsby page html resolver.
 */
class PageHtmlCustomUrlResolverController extends ControllerBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The Gatsby mocker service.
   *
   * @var \Drupal\static_preview_gatsby_instant\Mocker\GatsbyMocker
   */
  protected $gatsbyMocker;

  /**
   * The static builder plugin manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderManagerInterface
   */
  protected $staticBuilderManager;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user. For backwards compatibility this is optional, however
   *   this will be removed before Drupal 9.0.0.
   * @param \Drupal\static_preview_gatsby_instant\Mocker\GatsbyMocker $gatsby_mocker
   *   Gatsby mocker service.
   * @param \Drupal\static_build\Plugin\StaticBuilderManagerInterface $staticBuilderPluginManager
   *   The static builder plugin manager.
   */
  public function __construct(AccountInterface $current_user, GatsbyMocker $gatsby_mocker, StaticBuilderManagerInterface $staticBuilderPluginManager) {
    $this->currentUser = $current_user;
    $this->gatsbyMocker = $gatsby_mocker;
    $this->staticBuilderManager = $staticBuilderPluginManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('static_preview_gatsby_instant.gatsby_mocker'),
      $container->get('plugin.manager.static_builder')
    );
  }

  /**
   * Given a node url, return page data in Gatsby format.
   *
   * @param string $pagePath
   *   Page path with leading slash.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse|\Symfony\Component\HttpFoundation\Response
   *   JSON response on success, or Response with 404 code on error.
   */
  public function pageHtmlResolver(string $pagePath) {
    if ($pagePath) {
      // Restore path slashes.
      $pagePath = str_replace(":", "/", $pagePath);

      // Handle homepage case.
      if ($pagePath === "index") {
        $pagePath = '/';
      }

      $runningBuildData = [];
      if ($this->currentUser->isAuthenticated()) {

        $releaseManager = NULL;
        try {
          // We know its id from static_builder_gatsby, which is a dependency
          // of this module.
          $builderId = 'gatsby';
          /* @var $builder \Drupal\static_build\Plugin\StaticBuilderInterface */
          $builder = $this->staticBuilderManager->getInstance([
            'plugin_id' => $builderId,
            'configuration' => [
              'language-code' => $this->languageManager->getDefaultLanguage()->getId(),
              'run-mode' => StaticBuilderInterface::RUN_MODE_PREVIEW,
            ],
          ]);
          $releaseManager = $builder->getReleaseManager();
          $runningBuildData = $releaseManager->getTaskSupervisor()->getOngoingReleaseTaskData($builder->getTaskId());
        }
        catch (Throwable $e) {
          // Do nothing.
        }
      }

      $mockedPageHtml = $this->gatsbyMocker->getMockedPageHtml($pagePath, $runningBuildData);
      if ($mockedPageHtml) {
        return new Response($mockedPageHtml);
      }
    }
    // Throwing a NotFoundHttpException() caches that response, making it
    // impossible to serve the html once it's present.
    return new Response(NULL, 404, ['max-age' => 0]);
  }

}
