<?php

namespace Drupal\static_preview_gatsby_instant\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\Controller\NodeViewController as BaseNodeViewController;
use Drupal\Core\Entity\EntityInterface;
use Drupal\node\Entity\Node;
use Drupal\static_build\Plugin\StaticBuilderInterface;
use Drupal\static_build\Plugin\StaticBuilderManagerInterface;
use Drupal\static_preview_gatsby_instant\Mocker\GatsbyMocker;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Throwable;

/**
 * Defines a controller to render a single node.
 */
class PageHtmlNodeResolverController extends BaseNodeViewController {

  /**
   * Drupal config.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Gatsby mocker service.
   *
   * @var \Drupal\static_preview_gatsby_instant\Mocker\GatsbyMocker
   */
  protected $gatsbyMocker;

  /**
   * The static builder plugin manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderManagerInterface
   */
  protected $staticBuilderManager;

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user. For backwards compatibility this is optional, however
   *   this will be removed before Drupal 9.0.0.
   * @param \Drupal\static_preview_gatsby_instant\Mocker\GatsbyMocker $gatsbyMocker
   *   Gatsby mocker service.
   * @param \Drupal\static_build\Plugin\StaticBuilderManagerInterface $staticBuilderPluginManager
   *   The static builder plugin manager.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    EntityManagerInterface $entity_manager,
    LanguageManagerInterface $language_manager,
    RendererInterface $renderer,
    AccountInterface $current_user,
    GatsbyMocker $gatsbyMocker,
    StaticBuilderManagerInterface $staticBuilderPluginManager
  ) {
    parent::__construct($entity_manager, $renderer, $current_user);
    $this->configFactory = $configFactory;
    $this->languageManager = $language_manager;
    $this->gatsbyMocker = $gatsbyMocker;
    $this->staticBuilderManager = $staticBuilderPluginManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity.manager'),
      $container->get('language_manager'),
      $container->get('renderer'),
      $container->get('current_user'),
      $container->get('static_preview_gatsby_instant.gatsby_mocker'),
      $container->get('plugin.manager.static_builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function view(EntityInterface $node, $view_mode = 'full', $langcode = NULL) {
    // Avoid doing anything if module's config is empty.
    $config = $this->configFactory->get('static_preview_gatsby_instant.settings');
    if ($config->get('preview_component_path')) {
      if ($view_mode === 'full' && $node instanceof Node) {
        $builtContentTypes = $this->configFactory->get('static_build.settings')->get('built_content_types');
        if (in_array($node->bundle(), $builtContentTypes, TRUE)) {
          // Check if we must show an info message about a running build taking
          // place.
          $runningBuildData = [];
          if ($this->currentUser->isAuthenticated()) {

            $releaseManager = NULL;
            try {
              // We know its id from static_builder_gatsby, which is a
              // dependency of this module.
              $builderId = 'gatsby';
              $builder = $this->staticBuilderManager->getInstance([
                'plugin_id' => $builderId,
                'configuration' => [
                  'language-code' => $this->languageManager->getDefaultLanguage()->getId(),
                  'run-mode' => StaticBuilderInterface::RUN_MODE_PREVIEW,
                ],
              ]);
              $releaseManager = $builder->getReleaseManager();
              $runningBuildData = $releaseManager->getTaskSupervisor()->getOngoingReleaseTaskData($builder->getTaskId());
            }
            catch (Throwable $e) {
              // Do nothing.
            }
          }

          $pathInfo = \Drupal::requestStack()->getCurrentRequest()->getPathInfo();
          $mockedPageHtml = $this->gatsbyMocker->getMockedPageHtml($pathInfo, $runningBuildData);
          if ($mockedPageHtml) {
            return new Response($mockedPageHtml);
          }
        }
      }
    }

    return parent::view($node, $view_mode, $langcode);
  }

}
