<?php

namespace Drupal\static_preview_gatsby_instant\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\static_build\Plugin\StaticBuilderInterface;

/**
 * Configuration form for Static Preview - Gatsby - Instant.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'static_preview_gatsby_instant_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['static_preview_gatsby_instant.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('static_preview_gatsby_instant.settings');

    $form['preview_component_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Preview component path'),
      '#required' => TRUE,
      // TODO add path to preview release dir when everything is moved to
      //   Drupal config.
      '#description' => $this->t("It must start with a leading slash. Path to your Gatsby preview component, e.g.- /preview, inside Gatsby's public folder. Please refer to the docs for guidance on what should that component do."),
      '#default_value' => $config->get('preview_component_path'),
    ];

    // TODO remove this and move all "build_trigger_regexp_list" related config
    //  to static_build config, since it's there where it belongs and we
    //  shouldn't repeat it in every static_preview_* module.
    // TODO Add "build_trigger_regexp_list" for live and preview.
    $form['build_trigger_regexp_list'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Regular Expressions to detect changed files that trigger a "@preview" rebuild', ['@preview' => StaticBuilderInterface::RUN_MODE_PREVIEW]),
      '#required' => FALSE,
      '#description' => $this->t(
        "If a global part of your site is built with content coming from Drupal (e.g.- a global header or footer) it won't be updated unless a build is run. You should define, inside <a href='@static_build_config_form_link'>Static Build module's configuration form</a>, which files must trigger a '@preview' build.",
        [
          '@static_build_config_form_link' => Url::fromRoute('static_build.settings')->toString(),
          '@preview' => StaticBuilderInterface::RUN_MODE_PREVIEW,
        ]),
    ];

    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced'),
      '#open' => $config->get('custom_urls') ? TRUE : FALSE,
    ];

    $form['advanced']['custom_urls'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom URLs and data files'),
      '#required' => FALSE,
      '#description' => $this->t(
        "There are some cases where a data file is exported using a custom exporter not related to nodes. For such cases, you should define the relationship between URLs and data files. Enter one expression per line, defining a Regular Expression pattern for a URL (with a leading slash), a separator (space+three dashes+space, ' --- '), and a replacement for the previous pattern pointing to a data file (without a leading slash) inside Static Export's data directory (@static_export_data_dir), i.e.- ^/my-custom-path\/(.+) --- custom/path/$1.json",
        [
          "@static_export_data_dir" => $this->config("static_export.settings")->get("data_dir"),
        ]
      ),
      '#default_value' => $config->get('custom_urls') ? implode("\n", $config->get('custom_urls')) : NULL,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $previewPath = $form_state->getValue('preview_component_path');
    if (strpos($previewPath, '/') !== 0) {
      $form_state->setErrorByName(
        'preview_component_path',
        $this->t('Preview component path must start with a leading slash.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('static_preview_gatsby_instant.settings');

    // Explode, trim and remove empty values.
    $customUrlsArray = explode("\n", $form_state->getValue('custom_urls'));
    $customUrlsArray = array_map('trim', $customUrlsArray);
    // Remove empty values.
    $customUrlsArray = array_filter($customUrlsArray);

    $config
      ->set('preview_component_path', preg_replace("/\/$/", "", $form_state->getValue('preview_component_path')))
      ->set('custom_urls', $customUrlsArray)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
