<?php

namespace Drupal\static_preview_gatsby_instant\Mocker;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\static_build\Plugin\StaticBuilderInterface;
use Drupal\static_build\Plugin\StaticBuilderManagerInterface;
use Drupal\static_build\Release\ReleaseManagerInterface;
use Drupal\static_export\Service\NodePathResolverService;
use Drupal\static_preview_gatsby_instant\PathResolver\CustomUrlPathResolverInterface;

/**
 * A mocker service to provide Gatsby with mocked data, components, etc.
 */
class GatsbyMocker {

  /**
   * Drupal config.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The Language Manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The node path resolver service.
   *
   * @var \Drupal\static_export\Service\NodePathResolverService
   */
  protected $nodePathResolverService;

  /**
   * The static builder plugin manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderManagerInterface
   */
  protected $staticBuilderManager;

  /**
   * Custom URL path resolver.
   *
   * @var \Drupal\static_preview_gatsby_instant\PathResolver\CustomUrlPathResolverInterface
   */
  protected $customUrlPathResolver;

  /**
   * Release manager.
   *
   * @var \Drupal\static_build\Release\ReleaseManagerInterface
   */
  protected $releaseManager;

  /**
   * GatsbyMocker service constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\static_build\Plugin\StaticBuilderManagerInterface $staticBuilderManager
   *   The static builder manager.
   * @param \Drupal\static_export\Service\NodePathResolverService $nodePathResolverService
   *   Node path resolver.
   * @param \Drupal\static_preview_gatsby_instant\PathResolver\CustomUrlPathResolverInterface $custom_url_path_resolver
   *   Custom URL path resolver.
   *
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    LanguageManagerInterface $languageManager,
    StaticBuilderManagerInterface $staticBuilderManager,
    NodePathResolverService $nodePathResolverService,
    CustomUrlPathResolverInterface $custom_url_path_resolver
  ) {
    $this->configFactory = $configFactory;
    $this->languageManager = $languageManager;
    $this->staticBuilderManager = $staticBuilderManager;
    $this->nodePathResolverService = $nodePathResolverService;
    $this->customUrlPathResolver = $custom_url_path_resolver;
    $this->releaseManager = $this->getReleaseManagerFromPlugin();
  }

  /**
   * Given a node url, return mocked page data for preview component.
   *
   * @param string $pagePath
   *   Page path with leading slash.
   *
   * @return array
   *   The page data in array format, ready to be returned in a JsonResponse.
   */
  public function getMockedPageData(string $pagePath) {
    // $pagePath could be a node's path or a custom url.
    if ($this->customUrlPathResolver->isCustomUrl($pagePath)) {
      $absolutePath = $this->customUrlPathResolver->getExportedFilePathAbsolute($pagePath);
      $relativePath = $this->customUrlPathResolver->getExportedFilePathRelative($pagePath);
    }
    else {
      $absolutePath = $this->nodePathResolverService->getExportedFilePathAbsoluteByNodeUrl($pagePath);
      $relativePath = $this->nodePathResolverService->getExportedFilePathRelativeByNodeUrl($pagePath);
    }

    $pageDataContents = @file_get_contents($absolutePath);
    if (!empty($pageDataContents)) {
      $pageDataContentsArray = json_decode($pageDataContents, TRUE);
      $pageDataContentsArray['data']['content']['sourceFile'] = $relativePath;
      if (isset($pageDataContentsArray['data']['content']['url']['path'])) {
        $previewPageDataArray = $this->getPreviewComponentPageData();
        if (isset($previewPageDataArray['result']['pageContext'])) {
          // Hydrate page-data.json template with page's data.
          // Instead of using
          // $pageDataContentsArray['data']['content']['url']['path'], honor
          // $pagePath and set it to $previewPageDataArray['path']. This way,
          // accessing the same page from different aliases (like /node/XXX)
          // will always work.
          $previewPageDataArray['path'] = $pagePath[0] === "/" ? $pagePath : "/" . $pagePath;
          $previewPageDataArray['result']['pageContext']['node'] = $pageDataContentsArray;
          return $previewPageDataArray;
        }
      }
    }
    return NULL;
  }

  /**
   * Given a node id, return mocked page html for preview component.
   *
   * @param string $pagePath
   *   Node path info (route).
   * @param array $runningBuildData
   *   Running build data.
   *
   * @return string
   *   The mocked html.
   */
  public function getMockedPageHtml(string $pagePath, array $runningBuildData = []) {
    // Ensure $pathInfo always stars with a leading slash.
    if ($pagePath[0] !== '/') {
      $pagePath = '/' . $pagePath;
    }

    $previewHtml = $this->getPreviewComponentHtml();
    if ($previewHtml) {
      $previewData = 'window.pagePath="' . $pagePath . '";';
      if (count($runningBuildData) > 0) {
        $previewData .= 'window.GATSBY_INSTANT_PREVIEW___RUNNING_BUILD_DATA={';
        foreach ($runningBuildData as $key => $value) {
          $previewData .= str_replace('-', '_', $key) . ":";
          if ($value === NULL) {
            $previewData .= "null";
          }
          else {
            $previewData .= (is_numeric($value) ? $value : "'$value'");
          }
          $previewData .= ",";
        }
        $previewData .= '};';
      }

      $previewHtml = str_replace(
        // Replace original window.pagePath.
        [
          'window.pagePath="' . $this->getPreviewComponentPathFromConfig() . '";',
          '<link as="fetch" rel="preload" href="/page-data' . $this->getPreviewComponentPathFromConfig() . '/page-data.json"',
        ],
        // Replace preload of preview components's original page-data.json.
        [
          $previewData,
          '<link as="fetch" rel="preload" href="/page-data' . ($pagePath === "/" ? "/index" : $pagePath) . '/page-data.json"',
        ], $previewHtml
      );

      return $previewHtml;
    }
    return NULL;
  }

  /**
   * Get preview component's page-data.json to use it as a template.
   *
   * @return array|null
   *   The page data in array format.
   */
  protected function getPreviewComponentPageData() {
    $currentRelease = $this->releaseManager->getCurrentRelease();
    if ($currentRelease) {
      $previewPageDataPath = $currentRelease->getDir() . "/page-data" . $this->getPreviewComponentPathFromConfig() . "/page-data.json";
      $previewPageDataContents = @file_get_contents($previewPageDataPath);
      if (!empty($previewPageDataContents)) {
        $previewPageDataArray = json_decode($previewPageDataContents, TRUE);
        if (isset($previewPageDataArray['result']['pageContext'])) {
          $previewPageDataArray['result']['pageContext']['isCreatedByStatefulCreatePages'] = FALSE;
          return $previewPageDataArray;
        }
      }
    }
    return NULL;
  }

  /**
   * Get preview component's html to use it as a template.
   *
   * @return string|null
   *   Component's html.
   */
  protected function getPreviewComponentHtml() {
    $currentRelease = $this->releaseManager->getCurrentRelease();
    if ($currentRelease) {
      $previewHtmlPath = $currentRelease->getDir() . $this->getPreviewComponentPathFromConfig() . "/index.html";
      $previewHtmlContents = @file_get_contents($previewHtmlPath);
      if (!empty($previewHtmlContents)) {
        return $previewHtmlContents;
      }
    }
    return NULL;
  }

  /**
   * Get preview component path from config.
   *
   * @return string
   *   Preview component path from config.
   */
  protected function getPreviewComponentPathFromConfig() {
    $config = $this->configFactory->get('static_preview_gatsby_instant.settings');
    return $config->get('preview_component_path');
  }

  /**
   * Get release manager from plugin.
   *
   * @return \Drupal\static_build\Release\ReleaseManagerInterface
   *   The plugin's release manager.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\static_suite\StaticSuiteException
   */
  protected function getReleaseManagerFromPlugin(): ReleaseManagerInterface {
    if (!$this->releaseManager) {
      // We know its id is "gatsby" from static_builder_gatsby, which is a
      // dependency of this module.
      $builder = $this->staticBuilderManager->getInstance([
        'plugin_id' => 'gatsby',
        'configuration' => [
          'language-code' => $this->languageManager->getDefaultLanguage()->getId(),
          'run-mode' => StaticBuilderInterface::RUN_MODE_PREVIEW,
        ],
      ]);
      $this->releaseManager = $builder->getReleaseManager();
    }

    return $this->releaseManager;
  }

}
