<?php

namespace Drupal\static_preview_subset_build\Controller;

use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\static_build\Plugin\StaticBuilderInterface;
use Drupal\static_build\Plugin\StaticBuilderManagerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\static_export\Plugin\ExporterManagerInterface;
use Drupal\static_preview_subset_build\Preview\PreviewManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Throwable;

/**
 * Provides a preview system.
 */
class PreviewController extends ControllerBase {
  use StringTranslationTrait;

  /**
   * Drupal Language Manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The static builder plugin manager.
   *
   * @var \Drupal\static_build\Plugin\StaticBuilderManagerInterface
   */
  protected $staticBuilderManager;

  /**
   * The preview manager.
   *
   * @var \Drupal\static_preview_subset_build\Preview\PreviewManager
   */
  protected $previewManager;

  /**
   * Exporter Manager.
   *
   * @var \Drupal\static_export\Plugin\ExporterManagerInterface
   */
  protected $exporterManager;

  /**
   * Current language code.
   *
   * @var string
   */
  protected $languageCode;

  /**
   * PreviewController constructor.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   Drupal language manager.
   * @param \Drupal\core\Messenger\MessengerInterface $messenger
   *   Drupal messenger service.
   * @param \Drupal\static_build\Plugin\StaticBuilderManagerInterface $staticBuilderPluginManager
   *   The static builder manager.
   * @param \Drupal\static_preview_subset_build\Preview\PreviewManager $previewManager
   *   The preview manager.
   * @param \Drupal\static_export\Plugin\ExporterManagerInterface $exporterManager
   *   Exporter Manager.
   */
  public function __construct(
    LanguageManagerInterface $languageManager,
    MessengerInterface $messenger,
    StaticBuilderManagerInterface $staticBuilderPluginManager,
    PreviewManager $previewManager,
    ExporterManagerInterface $exporterManager
  ) {
    $this->languageManager = $languageManager;
    $this->messenger = $messenger;
    $this->staticBuilderManager = $staticBuilderPluginManager;
    $this->previewManager = $previewManager;
    $this->exporterManager = $exporterManager;

    $this->languageCode = $this->languageManager->getDefaultLanguage()->getId();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('language_manager'),
      $container->get('messenger'),
      $container->get('plugin.manager.static_builder'),
      $container->get('static_preview_subset_build.preview_manager'),
      $container->get('plugin.manager.static_exporter')
    );
  }

  /**
   * Triggers a preview batch operation.
   *
   * @param string $nid
   *   Node id to be previewed.
   * @param string|null $uniqueId
   *   The unique id of the export process which created this preview call.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   A render array or a RedirectResponse.
   */
  public function preview(string $nid, string $uniqueId = NULL) {
    $nid = intval($nid);

    // Ensure it can be previewed.
    if (!$this->previewManager->canBePreviewed($nid)) {
      $this->messenger->addError($this->t("Node @nid cannot be previewed. Either it's already published, or it's content type is not allowed to be previewed.", ["@nid" => $nid]));
      return $this->redirect("entity.node.canonical", ['node' => $nid], [], 301);
    }

    $localBuilders = $this->config('static_build.settings')->get('local.builders');
    $pluginId = array_shift($localBuilders);
    if (!$pluginId) {
      return [
        '#markup' => '<p>' . $this->t('No default local builder selected') . '</p>',
      ];
    }

    // Load the plugin.
    try {
      $plugin = $this->staticBuilderManager->getInstance([
        'plugin_id' => $pluginId,
        'configuration' => [
          'run-mode' => StaticBuilderInterface::RUN_MODE_PREVIEW,
        ],
      ]
      );
    }
    catch (Throwable $e) {
      return [
        '#markup' => "<p>" . $this->t("Unable to load plugin '@plugin_id'.", ["@plugin_id" => $pluginId]) . "</p>",
      ];
    }

    try {
      $this->previewManager->init($plugin);
    }
    catch (Throwable $e) {
      return [
        '#markup' => "<p>" . $this->t('Unable to initialize the preview manager') . "</p>",
      ];
    }

    // If there is a $uniqueId, it means that we are previewing a node that has
    // just been saved clicking on "Save & Preview" button.
    // If it's not that case:
    if (!$uniqueId) {
      // Check if preview is stale to redirect to preview file.
      if ($this->previewManager->isNodePreviewStale($nid)) {
        // Export it and get a uniqueId.
        try {
          $entityExporter = $this->exporterManager->getInstance(['plugin_id' => 'entity']);
          $entityExporter->setLanguageCode($this->languageCode);
          // Don't trigger a build because it will be
          // done later calling $plugin->init()
          $entityExporter->setMustBuild(FALSE);
          $entityExporter->setIsForceWrite(TRUE);
          $fileCollectionGroup = $entityExporter->export(
          [
            $entityExporter::PARAM_ENTITY_TYPE_ID => "node",
            $entityExporter::PARAM_ENTITY_ID => $nid,
          ]
          );
          $uniqueId = $fileCollectionGroup->getNewestFileCollection()->uniqueId();
        }
        catch (Throwable $e) {
          file_put_contents('/tmp/ERROR_PREVIEW_' . $nid . '_' . time() . '.log', 'ERROR' . $e->getMessage() . "\n" . $e->getTraceAsString());
          return [
            '#markup' => "<p>" . $this->t("Unable to export content:") . " '$nid'</p>",
          ];
        }
      }
      else {
        // Redirect to its Url.
        $url = Url::fromUserInput($this->previewManager->getNodeUrl($nid));
        return new RedirectResponse($url->toString());
      }
    }

    // Add this item to the preview group.
    try {
      $this->previewManager->addNodeToPreviewGroup($nid);
    }
    catch (Throwable $e) {
      file_put_contents('/tmp/ERROR_PREVIEW_' . $nid . '_' . time() . '.log', 'ERROR' . $e->getMessage() . "\n" . $e->getTraceAsString());
      return [
        '#markup' => "<p>" . ($this->t("Node @nid cannot be saved onto the preview group.", ["@nid" => $nid])) . "</p>",
      ];
    }

    // Trigger a background build.
    try {
      $plugin->init();
    }
    catch (Throwable $e) {
      return [
        '#markup' => "<p>" . $this->t("Preview error:") . " " . $e->getMessage() . " " . $e->getTraceAsString() . "</p>",
      ];
    }

    // Show progress.
    return [
      '#theme' => 'static_preview_subset_build_preview',
      '#unique_id' => $uniqueId,
      '#nid' => $nid,
      '#node_url' => $this->previewManager->getNodeUrl($nid),
      '#node_title' => $this->previewManager->getNodeTitle($nid),
      '#progress_label' => $this->t("Previewing"),
      '#progress_description_default' => $this->t("Connecting to the preview queue system. This may take some seconds..."),
      '#progress_description_running' => $this->t("Running a preview process for this content. When finished, you will be redirected to its preview."),
      '#progress_description_queue' => $this->t("Another preview process is running. Waiting for it to finish. Its current status is:"),
      '#error_title' => $this->t("Error message"),
      '#error_message' => $this->t("We are unable to preview this content at this moment. Please, <a href='/node/@nid/edit'>edit the content</a> and try again.", ['@nid' => $nid]),
      '#status_endpoint' => Url::fromRoute('static_build.releases_list.running_percentage', ['builderId' => $pluginId, 'runMode' => $plugin->getConfiguration()['run-mode']]),
      '#attached' => [
        'library' => [
          'static_preview_subset_build/preview-info',
        ],
      ],
    ];

  }

}
