<?php

namespace Drupal\static_suite\Entity;

use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\Plugin\Validation\Constraint\BundleConstraint;

/**
 * A set of useful utils related to entities.
 */
class EntityUtils implements EntityUtilsInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Entity bundle id cache array.
   *
   * @var array
   */
  protected $entityBundleIdCache = [];

  /**
   * Constructs the entity utils.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public function loadEntity(string $entityTypeId, string $entityId): ?EntityInterface {
    $storage = $this->entityTypeManager->getStorage($entityTypeId);
    return $storage->load($entityId);
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityIds(string $entityTypeId, $options = []): array {
    $storage = $this->entityTypeManager->getStorage($entityTypeId);
    $query = $storage->getQuery();

    $hasBundle = $storage->getEntityType()->getKey('bundle');
    if ($hasBundle === 'type' && !empty($options['bundle'])) {
      $query->condition('type', $options['bundle']);
    }

    $hasStatus = $storage->getEntityType()->getKey('status');
    if ($hasStatus && !empty($options['status'])) {
      $query->condition('status', $options['status']);
    }

    if (isset($options['conditions'])) {
      foreach ($options['conditions'] as $condition) {
        if (!in_array($condition['field'], ['status', 'type'])) {
          $query->condition($condition['field'], $condition['value'], $condition['operator'] ?? NULL);
        }
      }
    }

    if (isset($options['sort'])) {
      $query->sort($options['sort']['field'], $options['sort']['direction']);
    }

    if (isset($options['range'])) {
      $query->range($options['range']['start'], $options['range']['length']);
    }

    return $query->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityBundleId(EntityInterface $entity): ?string {
    if (!isset($this->entityBundleIdCache[$entity->id()])) {
      $entityBundleId = NULL;
      $constrains = $entity->getTypedData()->getConstraints();
      foreach ($constrains as $constrain) {
        if ($constrain instanceof BundleConstraint) {
          if (is_array($constrain->bundle)) {
            $entityBundleId = $constrain->bundle[0];
          }
          else {
            $entityBundleId = $constrain->bundle;
          }
          break;
        }
      }
      $this->entityBundleIdCache[$entity->id()] = $entityBundleId;
    }
    return $this->entityBundleIdCache[$entity->id()];
  }

  /**
   * {@inheritdoc}
   */
  public function hasStatusChanged(EditorialContentEntityBase $entity): bool {
    $statusHasChanged = FALSE;
    $entityOldStatus = $entity->isPublished();
    if (is_object($entity->original) && method_exists($entity->original, 'isPublished')) {
      $entityOldStatus = $entity->original->isPublished();
    }
    $entityNewStatus = $entity->isPublished();
    if ($entityOldStatus !== $entityNewStatus) {
      $statusHasChanged = TRUE;
    }
    return $statusHasChanged;
  }

  /**
   * {@inheritdoc}
   */
  public function hasKey(EditorialContentEntityBase $entity): bool {
    $statusHasChanged = FALSE;
    $entityOldStatus = $entity->isPublished();
    if (is_object($entity->original) && method_exists($entity->original, 'isPublished')) {
      $entityOldStatus = $entity->original->isPublished();
    }
    $entityNewStatus = $entity->isPublished();
    if ($entityOldStatus !== $entityNewStatus) {
      $statusHasChanged = TRUE;
    }
    return $statusHasChanged;
  }

}
